
%
% ImarisCell Custom Statistic for Change in Area 8.4.1
%
%  Copyright Dan Shiwarski 2017
%
%
%  Installation:
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Biomechanical Analysis">
%        <Item name="Dilation from Change in Cell Area" icon="Matlab">
%          <Command>MatlabXT::XTCellAreaDilation(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpCells">
%         <Item name="Create Statistic for Change in Area" icon="Matlab" tooltip="Calculate the change in area as dilation for cells.">
%            <Command>MatlabXT::XTCellAreaDilation(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
%
% !!! Additional informations:
% Original dataset size is maintained
% function XTCellAreaDilation(aImarisApplicationID)


function XTCellAreaDilation(aImarisApplicationID)

%Allow for user to click the cells so that Imaris knows which ones
%you're interested in
f = warndlg('Select the cell objects that have been added to Imaris, then click OK');
waitfor(f)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Cells Objects and Data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFactory = vImarisApplication.GetFactory;
vCells = vFactory.ToCells(vImarisApplication.GetSurpassSelection);% Need to have the cells objected selected in surpass view
vCellsIds = vCells.GetIds;
vCellsTrackIds = vCells.GetTrackIds;
vCellsTrackEdges = vCells.GetTrackEdges;

% Counting number of cells
vCountCells = vCells.GetNumberOfCells;


% Getting the Statistics 
    vAllStatistics = vCells.GetStatistics;
    vNames       = cell(vAllStatistics.mNames);
    vValues      = vAllStatistics.mValues;
    vUnits       = cell(vAllStatistics.mUnits); % not used
    vFactors     = cell(vAllStatistics.mFactors);
    vFactorNames = cellstr(char(vAllStatistics.mFactorNames));
    vIds         = vAllStatistics.mIds;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Name of the Similarity Statistics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vPrompt      = 'Enter the name of the strain statistic (Dilation prefix will be added)';
vDlgTitle    = 'Strain Statistics Name';
vNumOfLines  = 1;
vDefaultName = {''};
vStrainStatisticsName = char(inputdlg(vPrompt,vDlgTitle,vNumOfLines,vDefaultName));
if isempty(vStrainStatisticsName), 
    return, 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extracting the track Ids and the Ids to build
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

vStatisticCellArea = find(strcmp('Cell Area', vNames)); 
vExtractedCellArea = double(vValues(vStatisticCellArea));      
    
vStatisticObjectId = find(strcmp('Cell Time Index', vNames)); 
vExtractedObjectIds = double(vIds(vStatisticObjectId));   
    
vStatisticId = find(strcmp('Cell Time Index', vNames));      
vExtractedCellIds = double(vIds(vStatisticId));   
    
vStatisticTime = find(strcmp('Cell Time Index', vNames)); 
vExtractedTime = double(vValues(vStatisticTime));   

CellAreaMatrix = [vExtractedCellArea, vExtractedTime, vExtractedCellIds, vExtractedObjectIds];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Importing tracked data for cells with the data
% sorted by TrackID then by Time point 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SortedDilationMatrix = MeshCellDilation(CellAreaMatrix,vCellsTrackEdges); % These are independent functions needed to accompany this script 
[row,column] = size(SortedDilationMatrix);% Calculating the size [rows,columns] of data matrix

%%%%%%%%%%%%%%%%%%%%%%%%%
% Set New Track Statistic
%%%%%%%%%%%%%%%%%%%%%%%%%
vIndT=1:length(SortedDilationMatrix(:,3));% Total number of Track IDs
vIdsT= SortedDilationMatrix(:,3)';% Extracting the Cell ID #'s
vUnitsT(vIndT) = {'Area Dilation'};% Setting the units 

% Creating vFactors string array
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling the Imaris object description 
% Loop to build the additional rows in the string for vFactorT 
% that would contain Channel information,collection, and time. These 
% categories need to match the original vFactors that are found when 
% exporting all statistics.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFactorsT(vIndT) = {'Cell'};% Row 1 of vFactorsT is the Object 'Cell' for all 
    for i = 1:row
        vFactorsT(2,i) = {''};% Blank rows 
        vFactorsT(3,i) = {num2str(SortedDilationMatrix(i,2))};% Extracting the time points from SortedStrainMatrix and making a string 
        vFactorsT(4,i) = {''};% Blank rows (This stands for 0 since it is of type dendrite)
    end

% Creating Remaining string arrays
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vNamesT(vIndT) = {['Dilation',' ',vStrainStatisticsName]};%Statistic name that will appear in the dropdown menu
vFactorNamesT = {'Category','Collection','Time','Type',};
vDilationStatistic = SortedDilationMatrix(:,1)';


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Exporting the New Statistic to Imaris
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vCells.AddStatistics(vNamesT, vDilationStatistic, vUnitsT, vFactorsT, vFactorNamesT, vIdsT); 

end

function [normalized] = MeshCellDilation(CellAreaMatrix,vCellsTrackEdges)

%assign trackID to appropriate locations in the matrix
%subtract 1 to account for fencepost problem (edges are one fewer in number
%than points). This is why there are -2s in certain places
vNumCells = numel(find(CellAreaMatrix(:,2)==1));
vNumTimePoints = max(CellAreaMatrix(:,2));
firstObjectID = CellAreaMatrix(1,4);
normalized = CellAreaMatrix;



%iterate through the Cells
for i = 1:vNumCells
    vCellsTrackEdgesShifted = vCellsTrackEdges(((i-1)*vNumTimePoints)+2-i:(i)*vNumTimePoints-i,1);%Get the edges for all time points for this filament  
    vCellsTrackEdgesShifted =[vCellsTrackEdgesShifted; vCellsTrackEdges((i)*vNumTimePoints-i,2)];%Append the final object ID for the filament
    
    setFirstObjectID=vCellsTrackEdgesShifted(1)+firstObjectID;%get the first object ID for this set of filaments
    firstObjectRowInFilLengthMatrix = CellAreaMatrix(find(CellAreaMatrix(:,4)==setFirstObjectID),:);%get the row corresponding to the first object ID
    initialArea = firstObjectRowInFilLengthMatrix(1);%get the initial length for this set of filaments
    %iterate through this set of filaments
    for j = 1:length(vCellsTrackEdgesShifted)
        currentObjectID=vCellsTrackEdgesShifted(j)+firstObjectID
        currentRowNumber = find(CellAreaMatrix(:,4)==currentObjectID)
        currentRow = CellAreaMatrix(currentRowNumber,:)
        currentDilation = (currentRow(1)-initialArea)/initialArea;
        normalized(currentRowNumber,1) = currentDilation;
    end
end

end
