
%
% ImarisCell Custom Statistic for Change in Area 8.4.1
%
%  Copyright Dan Shiwarski 2017
%
%
%  Installation:
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Biomechanical Analysis">
%        <Item name="Strain from Change in Filament Length" icon="Matlab">
%          <Command>MatlabXT::XTFilamentLengthStrain(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpCells">
%         <Item name="Create Statistic for Strain from Change in Length" icon="Matlab" tooltip="Calculate the change in length as strain for filaments.">
%            <Command>MatlabXT::XTFilamentLengthStrain(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
%
% !!! Additional informations:
% Original dataset size is maintained
% function XTFilamentLengthStrain(aImarisApplicationID)


function XTFilamentLengthStrain(aImarisApplicationID)

%Allow for user to click the cells so that Imaris knows which ones
%you're interested in
f = warndlg('Select the filament objects that have been added to Imaris, then click OK');
waitfor(f)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Filaments Objects and data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);% Need to have the filaments objected selected in surpass view
vFilamentsIds = vFilaments.GetIds;
vFilamentsTrackIds = vFilaments.GetFilamentTrackIds;
vFilamentsTrackEdges = vFilaments.GetFilamentTrackEdges;

% Counting number of cells
vCountFilaments = vFilaments.GetNumberOfFilaments;

% Getting the Statistics 
    vAllStatistics = vFilaments.GetStatistics;
    vNames       = cell(vAllStatistics.mNames);
    vValues      = vAllStatistics.mValues;
    vUnits       = cell(vAllStatistics.mUnits); % not used
    vFactors     = cell(vAllStatistics.mFactors);
    vFactorNames = cellstr(char(vAllStatistics.mFactorNames));
    vIds         = vAllStatistics.mIds;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Name of the Similarity Statistics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vPrompt      = 'Enter the name of the strain statistic (Strain prefix will be added)';
vDlgTitle    = 'Strain Statistics Name';
vNumOfLines  = 1;
vDefaultName = {''};
vStrainStatisticsName = char(inputdlg(vPrompt,vDlgTitle,vNumOfLines,vDefaultName));
if isempty(vStrainStatisticsName) 
    return 
end


% Extracting the track Ids and the Ids to build

vStatisticDendriteLength = find(strcmp('Dendrite Length', vNames)); 
vExtractedDendriteLength = double(vValues(vStatisticDendriteLength));      
    
vStatisticObjectId = find(strcmp('Filament Time Index', vNames)); 
vExtractedObjectIds = double(vIds(vStatisticObjectId));   
    
vStatisticId = find(strcmp('Dendrite Time Index', vNames));      
vExtractedDendriteIds = double(vIds(vStatisticId));   
    
vStatisticTime = find(strcmp('Dendrite Time Index', vNames)); 
vExtractedTime = double(vValues(vStatisticTime));   

FilamentLengthMatrix = [vExtractedDendriteLength, vExtractedTime, vExtractedDendriteIds, vExtractedObjectIds];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Importing an Excel Sheet with the tracked data for cells with the data
% sorted by TrackID then by Time point 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SortedStrainMatrix = MeshStrainLength(FilamentLengthMatrix,vFilamentsTrackEdges); % These are independent functions needed to accompany this script 
[row,column] = size(SortedStrainMatrix);% Calculating the size [rows,columns] of data matrix

%%%%%%%%%%%%%%%%%%%%%%%%%
% Set New Track Statistic
%%%%%%%%%%%%%%%%%%%%%%%%%
vIndT=1:length(SortedStrainMatrix(:,3));% Total number of Track IDs
vIdsT= SortedStrainMatrix(:,3)';% Extracting the Dendrite ID #'s
vUnitsT(vIndT) = {'strain'};% Setting the units 

% Creating vFactors string array
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling the Imaris object description 
% Loop to build the additional rows in the string for vFactorT 
% that would contain Channel information,collection, and time. These 
% categories need to match the original vFactors that are found when 
% exporting all statistics.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFactorsT(vIndT) = {'Dendrite'};% Row 1 of vFactorsT is the Object 'Cell' for all 
    for i = 1:row
        vFactorsT(2,i) = {''};% Blank rows 
        vFactorsT(3,i) = {num2str(SortedStrainMatrix(i,2))};% Extracting the time points from SortedStrainMatrix and making a string 
        vFactorsT(4,i) = {''};% Blank rows (This stands for 0 since it is of type dendrite)
    end

% Creating Remaining string arrays
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vNamesT(vIndT) = {['Strain',' ',vStrainStatisticsName]};%Statistic name that will appear in the dropdown menu
vFactorNamesT = {'Category','Collection','Time','Type',};
vStrainStatistic = SortedStrainMatrix(:,1)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Exporting the New Statistic to Imaris
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%vFilaments.AddStatistics(vNamesT, vStrainStatistic, vUnitsT, vFactorsT, vFactorNamesT, vIdsT); 
vFilaments.AddStatistics(vNamesT, vStrainStatistic, vUnitsT, vFactorsT, vFactorNamesT, FilamentLengthMatrix(:,3)); 

end

function [normalized] = MeshStrainLength(FilamentLengthMatrix,vFilamentsTrackEdges)

%assign trackID to appropriate locations in the matrix
%subtract 1 to account for fencepost problem (edges are one fewer in number
%than points). This is why there are -2s in certain places
vNumFilaments = numel(find(FilamentLengthMatrix(:,2)==1));
vNumTimePoints = max(FilamentLengthMatrix(:,2));
firstObjectID = FilamentLengthMatrix(1,4);
normalized = FilamentLengthMatrix;

%iterate through the filaments
for i = 1:vNumFilaments
    vFilamentsTrackEdgesShifted = vFilamentsTrackEdges(((i-1)*vNumTimePoints)+2-i:(i)*vNumTimePoints-i,1);%Get the edges for all time points for this filament  
    vFilamentsTrackEdgesShifted =[vFilamentsTrackEdgesShifted; vFilamentsTrackEdges((i)*vNumTimePoints-i,2)];%Append the final object ID for the filament
    
    setFirstObjectID=vFilamentsTrackEdgesShifted(1)+firstObjectID;%get the first object ID for this set of filaments
    firstObjectRowInFilLengthMatrix = FilamentLengthMatrix(find(FilamentLengthMatrix(:,4)==setFirstObjectID),:);%get the row corresponding to the first object ID
    initialLength = firstObjectRowInFilLengthMatrix(1);%get the initial length for this set of filaments
    %iterate through this set of filaments
    for j = 1:length(vFilamentsTrackEdgesShifted)
        currentObjectID=vFilamentsTrackEdgesShifted(j)+firstObjectID
        currentRowNumber = find(FilamentLengthMatrix(:,4)==currentObjectID)
        currentRow = FilamentLengthMatrix(currentRowNumber,:)
        currentStrain = (currentRow(1)-initialLength)/initialLength;
        normalized(currentRowNumber,1) = currentStrain;
    end
end
% 
% %[sorted,sortOrder] = sort(FilamentLengthMatrix(:,3));%sort the matrix by ID
% sortedCellLengthMatrix = FilamentLengthMatrix(sortOrder,:);%sort the matrix by ID
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Get the Number of Time Points Per Filament
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% vNumTimePoints = max(FilamentLengthMatrix(:,2));
% vNumFilaments = numel(find(FilamentLengthMatrix(:,2)==1));
% 
% vLengthNormalized = zeros(vNumTimePoints,1); % Creating an empty matrix to append the normalized data to
% 
% % for i = 1: vNumTimePoints: (length(sortedCellLengthMatrix(:,1)))
% %     currentArray = sortedCellLengthMatrix(i:i+(vNumTimePoints-1),:); % Looking at all rows, column 3 of cellAreaMatrix in steps of Number of Time Points
% %     currentArray(:,1) = currentArray(:,1)./currentArray(1,1); % All rows first column of current Array is equal to all rows first column of current Array divided by the first row first column of currentArray
% %     vLengthNormalized = [vLengthNormalized;currentArray]; % creating normalized as the final data matrix and appending each itteration of the loop from currentArray to the normalized data matrix
% % end
% 
% ReferenceArray = sortedCellLengthMatrix(1:1+(vNumFilaments-1),1);
% 
% for j = 1:vNumFilaments
%     for i = j: vNumFilaments: (length(sortedCellLengthMatrix(:,1)))
%         currentNum = sortedCellLengthMatrix(i,1); % Looking at all rows, column 3 of cellAreaMatrix in steps of Number of Time Points
%         currentNumNormalized = currentNum/ReferenceArray(j);
%         vLengthNormalized(i) = currentNumNormalized; % creating normalized as the final data matrix and appending each itteration of the loop from currentArray to the normalized data matrix
%     end
% end
% 
% normalized = [vLengthNormalized sortedCellLengthMatrix(:,2:end)]; 

end