
%
%  Adding Filaments as segments between Nodes of Mesh for Strain
%  visulization 
%
%  Copyright Josh Tashman and Dan Shiwarski CMU 2018
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Biomechanical Analysis">
%        <Item name="Filaments and Strain Statistic Between Tracked Spots" icon="Matlab" tooltip="Adding Filaments as segments between spot nodes and calculating a strain statistic over time.">
%          <Command>MatlabXT::XTSpotsToFilamentsWithStrain(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSpots">
%          <Item name="Filaments and Strain Statistic Between Tracked Spots" icon="Matlab" tooltip="Adding Filaments as segments between spot nodes and calculating a strain statistic over time.">
%            <Command>MatlabXT::XTSpotsToFilamentsWithStrain(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
% 
%  Description:
%   
%   Taking the calculated legth between each paired node on the mesh that
%   has been tracked with spots over time and evaluated for change in
%   length as strain, and importing individual tracked filaments for each
%   segment to be visulaized in Imaris over time. Each segemtn is
%   represented as its own filament for tracking purposes, and has an 
%   individually calculated strain satistic. 
%

function imarisInput = XTSpotsToFilamentsWithStrain(aImarisApplicationID)

clc
close all
%Grab the points from Imaris
findPointsInput = ExportSpotsXYZtoMatlab(aImarisApplicationID);

%Get the skeletonized version of the thesholded mesh image
prompt = {'Please Select the skeletonized mask (skeleton must be white)'};
dlg_title = 'Please Select the skeletonized mask (skeleton must be white)';
num_lines = 0;
answer = inputdlg(prompt,dlg_title,num_lines);

[skelMeshFileName,PathName,FilterIndex] = uigetfile("*.*");
skelImg = imcomplement(imbinarize(imread([PathName skelMeshFileName])));

%Get the thresholded mesh image
prompt = {'Please Select the  NMBS reference image'};
dlg_title = 'Please Select the NMBS reference image';
num_lines = 0;
answer = inputdlg(prompt,dlg_title,num_lines);

[threshMeshFileName,PathName,FilterIndex] = uigetfile("*.*");
threshImg = (imread([PathName threshMeshFileName]));

%Get the tracking data for the image
prompt = {'What time point are the threshold and skeleton from (0 is the first time point possible)?'};
dlg_title = 'What time point are the threshold and skeleton from (0 is the first time point possible)?';
num_lines = 1;
trackingTimePoint = inputdlg(prompt,dlg_title,num_lines);

%Get the mesh spacing
prompt = {'What is the mesh spacing in microns?'};
dlg_title = 'What is the mesh spacing in microns?';
num_lines = 1;
meshSpacingMicrons = inputdlg(prompt,dlg_title,num_lines);
meshSpacingPixels = str2double(meshSpacingMicrons)/findPointsInput(1,7);%convert the mesh spacing into pixels

%Get the point relocation scalar
prompt = {'The skeleton points have to be relocated, what scalar would you use (default=.75)'};
dlg_title = 'What is the  search radius scalar?';
num_lines = 1;
defaultans = {num2str(.75)};
relocationRadiusScalarText = inputdlg(prompt,dlg_title,num_lines, defaultans);

%Run the initial script to get connections
relocationRadiusScalar = str2num(relocationRadiusScalarText{1});
[connectionArray, nodeStructArray] = findConnectionsBetweenSpots(skelImg, findPointsInput(findPointsInput(:,4)==str2double(trackingTimePoint{1}),:), threshImg, meshSpacingPixels, relocationRadiusScalar);

%Run the GUI for editing the automatically generated connections
[newConnections, nodeStructArray] = editConnectionsBetweenSpots(threshImg, connectionArray, nodeStructArray);

%Use the connections and node information as well as tracking information
%and a chosen number of points in a line to generate the lines
numberOfPointsInLine = 2;
[lineMatrix, numLineArray] = generateFilamentMatrix(newConnections, nodeStructArray, findPointsInput, numberOfPointsInLine);

%Format the line information into the imaris input format
imarisInput = generateImarisFilamentInput(lineMatrix, 2);

%Import Lines to Imaris
numTimePoints = length(lineMatrix);
AddCustomFilaments(aImarisApplicationID, imarisInput, numLineArray ,numberOfPointsInLine , numTimePoints)

%Allow for user to click the filaments so that Imaris knows which ones
%you're interested in
f = warndlg('Select the filaments that have been added to Imaris, then click OK');
waitfor(f)

%Import Strain Statistic to IMaris
ImportFilamentStrainStatistic(aImarisApplicationID)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions Required for XTSpotsToFilamentsWithStrain
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function findPointsInput = ExportSpotsXYZtoMatlab(aImarisApplicationID)
% 
%  Description:
%   
%   This function exports positional and tracking information for spots
%   tagged in imaris images to a matlab matrix with as many rows as tagged 
%   spots and 9 columns per spot. The output is findPointsInput into the
%   Matlab workspace.
%
%   The columns are organized as follows:
%
%   X Y Z time TrackID ID XpixScaled YpixScaled ZpixScaled 
% 

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
   javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;

  % vImarisApplication = vImarisLib.GetApplication(vImarisLib.GetServer.GetObjectID(0));
  
end

%% The following MATLAB code gets the maximum extend of the volume along XYZ
aExtendMaxX = vImarisApplication.GetDataSet.GetExtendMaxX;
aExtendMaxY = vImarisApplication.GetDataSet.GetExtendMaxY;
aExtendMaxZ = vImarisApplication.GetDataSet.GetExtendMaxZ;
aExtendMinX = vImarisApplication.GetDataSet.GetExtendMinX;
aExtendMinY = vImarisApplication.GetDataSet.GetExtendMinY;
aExtendMinZ = vImarisApplication.GetDataSet.GetExtendMinZ;

%% The following MATLAB code gets the dimension of the dataset along X
aSizeX = vImarisApplication.GetDataSet.GetSizeX;
aSizeY = vImarisApplication.GetDataSet.GetSizeY;
aSizeZ = vImarisApplication.GetDataSet.GetSizeZ;

%Determine the scale factor for physical dimensions to pixel dimensions
scaleX = (aExtendMaxX-aExtendMinX)/aSizeX;
scaleY = (aExtendMaxY-aExtendMinY)/aSizeY;
scaleZ = (aExtendMaxZ-aExtendMinZ)/aSizeZ;

% The Spots object is assumed to be selected in the Surpass Scene tree.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Spots Objects and data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vSpots = vImarisApplication.GetFactory.ToSpots(vImarisApplication.GetSurpassSelection);

%Extract information using vSpots object
vSpotValues = vSpots.Get;
vSpotsPositions = vSpotValues.mPositionsXYZ;
vSpotsIndicesT = vSpotValues.mIndicesT;
vSpotsTrackID = vSpots.GetTrackIds;
vSpotsTrackEdges = vSpots.GetTrackEdges;
%% mRadii gives only one radius per spot (and not one radius per axis).
%% vSpots.GetRadiiXYZ() function needs to be called in order to have the [x y z]-Radii.
vSpotsRadii = vSpotValues.mRadii; 
vSpotsIDs = vSpotValues.mIds; 

%Generate matrix equivalent to IMARIS excel export
findPointsInput = zeros(length(vSpotsPositions),9);
findPointsInput(:,1:3) = vSpotsPositions;
findPointsInput(:,4) = vSpotsIndicesT;
findPointsInput(:,6) = vSpotsIDs;
findPointsInput(:,7) = scaleX;
findPointsInput(:,8) = scaleY;
findPointsInput(:,9) = scaleZ;
            
numTimePoints = max(vSpotsIndicesT)+1;%add 1 to account for fencepost problem (edges are one fewer in number than points)


%assign trackID to appropriate locations in the matrix
%subtract 1 to account for fencepost problem (edges are one fewer in number
%than points). This is why there are -2s in certain places
uniqueTrackID = unique(vSpotsTrackID);
shiftedTrackEdges = vSpotsTrackEdges+1;
for i = 1:length(unique(vSpotsTrackID))
    currentIndex=i;
    currentEdgeIndex = find(shiftedTrackEdges(:,1)==currentIndex);%Where in track edges is the first column equal to i
    currentTrackID = uniqueTrackID(i);%Get the current track id
    findPointsInput(shiftedTrackEdges(currentEdgeIndex,1),5) = currentTrackID;%set the first instance's track id
    
    %Iterate through the track edges
    for j =1:(length(find(vSpotsTrackID(:,1)==currentTrackID))-1)%move from 1 to the total number of edges
        currentEdgeIndex = currentEdgeIndex + 1;%since the track edges for a given trackid are in row order, just add one each time
%         currentEdgeIndex = shiftedTrackEdges(currentIndex,1);%pull the location in the coordinate list that connects to this edge
        findPointsInput(shiftedTrackEdges(currentEdgeIndex,1),5) = currentTrackID;%set the track ID in the final data to that location
    end
    %The last position is in the second column, because these are edges
    findPointsInput(shiftedTrackEdges(currentEdgeIndex,2),5) = currentTrackID;%set the track ID in the final data to that location

end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%TRACKING INFORMATION MUST BE SORTED BY NODE ID
function [connectionArray, nodeStructArray] = findConnectionsBetweenSpots(skelImage, trackingInformation, mask, meshSpacingPixels, relocationRadiusScalar)
nodeStructArray = makeNodeStruct(trackingInformation);
connectionArray = generateConnectionArray(nodeStructArray);

%Edit the points to be on the skeletonized image
meshColor = 0;%0 for black mesh
[newNodes, oldNodes] = skelMeshPointsModify(skelImage, nodeStructArray, meshColor, meshSpacingPixels, relocationRadiusScalar);

%Display the mask and the tagged points
figure
imshow(mask,[])
hold on

for h = 1:length(newNodes)    
currentNewNode = newNodes(h);
currentOldNode = oldNodes(h);
scatter(currentNewNode.nodeX,currentNewNode.nodeY,'xr');
scatter(currentOldNode.nodeX,currentOldNode.nodeY,'og');
end
pause(.01);
visitedPoints = double(skelImage);

%pause;
%Choose where to start searching and how wide the radius of search should
%be
startNode = newNodes(1);
previousNode = startNode.nodeID;
xLoc = floor(startNode.nodeX);
yLoc = floor(startNode.nodeY);
%searchRadius = 11;
searchRadius = floor(.5*meshSpacingPixels);

%Begin connecting points
[connectionArray, previousNode, visitedPoints] = escape(previousNode, xLoc, yLoc, connectionArray, skelImage, newNodes, visitedPoints, searchRadius);

% %Plot the initial results
%        try     
%             imshow(mask,[]);
%             lines = [];
%             hold on
%             plot(nodeStructArray(previousNode).nodeX,nodeStructArray(previousNode).nodeY, '*m');
%             for k = 1:length(nodeStructArray)
%                 plotConnections = connectionArray(k).connections;
%                 currentNode = nodeStructArray(k);
%                 scatter(currentNode.nodeX,currentNode.nodeY,'UserData',currentNode.nodeID);
%                 if(~isempty(plotConnections))
%                     for j = 1:length(plotConnections)
%                         attachingNode = nodeStructArray(plotConnections(j));
%                         lines = [lines line([currentNode.nodeX attachingNode.nodeX],[currentNode.nodeY attachingNode.nodeY], 'UserData', [k j])];
%                     end
%                 end
%             end
%        catch
%            connectionArray;
%            nodeStructArray;
%        end
end

%PreviousNode is the last tagged node we were at
%xLoc is the new image x location
%yLoc is the new image y location
%connectionArray is an array of structures with connections already made between nodes
%image is a structure with image data and information about the image
%nodes is the structure array of tagged nodes
function [connectionArray, previousNode, visitedPoints] = escape(previousNode, xLoc, yLoc, connectionArray, image, nodeStructArray, visitedPoints,searchRadius)
    %Am I within the constraints of the image?
    [R, C] = size(image);
    if(xLoc<=C && xLoc>0 && yLoc<=R && yLoc>0)
        
        if(~isempty(nodeStructArray))
            xtemp = [];
            ytemp = [];
            for i = 1:length(nodeStructArray())
                xtemp = [xtemp floor(nodeStructArray(i).nodeX)];
                ytemp = [ytemp floor(nodeStructArray(i).nodeY)];
            end

            pixelValues = zeros(length(xtemp),1);

            for i = 1:length(xtemp)
                pixelValues(i) = visitedPoints(ytemp(i),xtemp(i));
            end

            if(sum(pixelValues)==(length(xtemp)-1))
                visitedPoints(ytemp,xtemp) = 0;
            end
        end
       
        %I've been everywhere
        if(sum(sum(visitedPoints))==(R*C))%blackmesh
        
        %I've been here
        elseif(visitedPoints(yLoc,xLoc)==1)%blackmesh
            
        %I haven't been here
        else  
            %I've been here now
            visitedPoints(yLoc,xLoc)=1;%blackmesh
                                 
            %Am I close to a node that isn't the node that I came from?
            distanceArray = generateDistanceArray(xLoc,yLoc,nodeStructArray);
            [sortDistanceArray, sortI] = sort(distanceArray);
        
            %See if there's a point less than radius pixels away & isn't the node we
            %just came from.
            if(sortDistanceArray(1)<searchRadius && sortI(1) ~= previousNode)
                previousNodeConnections = connectionArray(previousNode).connections;%get the connections that already exist for the previous node
            
                %iterate through those connections to see if this close node is
                %a new connection to the last node I came from
                connectionExists = false;
                for j = 1:length(previousNodeConnections)
                    if(previousNodeConnections(j) == sortI(1))
                        connectionExists = true;
                    end
                end
                
                %if the connection is new append it to the connection array for
                %the previousNode and update the previousNode
                if(~connectionExists)
                    connectionArray(previousNode).connections = [connectionArray(previousNode).connections sortI(1)];
                    connectionArray(sortI(1)).connections = [connectionArray(sortI(1)).connections previousNode];                   
                    
                    previousNode = sortI(1);
                    previousNodeX = floor(nodeStructArray(previousNode).nodeX);
                    previousNodeY = floor(nodeStructArray(previousNode).nodeY);      
                end
            end
            
            %Move out recursively to all surrounding points           
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc+1, yLoc, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc-1, yLoc, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc, yLoc+1, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc, yLoc-1, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc+1, yLoc+1, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc+1, yLoc-1, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc-1, yLoc+1, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
            [connectionArray, ~, visitedPoints]=escape(previousNode, xLoc-1, yLoc-1, connectionArray, image, nodeStructArray, visitedPoints, searchRadius);
           
        end
    end
end

%Creates an array of structures for storing the connections between nodes
function connectionArray = generateConnectionArray(nodeStructArray)
    for i = 1:length(nodeStructArray)
       currentConnection.connections = [];
       currentConnection.ID = nodeStructArray(i).nodeID;
       connectionArray(i) = currentConnection;
    end
end
    
%Generates an array of the distances between the point xLoc,yLoc and all the tacked nodes 
function distanceArray = generateDistanceArray(xLoc,yLoc,nodes)
    distanceArray = zeros(length(nodes),1);
    for i = 1:length(nodes)
        distanceArray(i) = sqrt((xLoc-nodes(i).nodeX)^2+(yLoc-nodes(i).nodeY)^2);
    end
end

%Make an array of structures using the tracking information to scale to
%pixel space
function nodeStructArray = makeNodeStruct(trackingInformation)
nodeStructArray = [];
x = (trackingInformation(:,1))./trackingInformation(1,7);
y = (trackingInformation(:,2))./trackingInformation(1,8);
z = (trackingInformation(:,3))./trackingInformation(1,9);
ID = (trackingInformation(:,5));

for i = 1:length(x)
    currentNodeStruct.nodeX = x(i);
    currentNodeStruct.nodeY = y(i);
    currentNodeStruct.nodeZ = z(i);
    currentNodeStruct.nodeID = i;
    currentNodeStruct.trackID = ID(i);
    nodeStructArray = [nodeStructArray currentNodeStruct];
end
end

%Moves the tagged points onto the closest pixel with the color of the mesh
function [newNodes, oldNodes] = skelMeshPointsModify(skelMesh, nodes, meshColor, meshSpacingPixels, relocationRadiusScalar)

%Save the original input
oldNodes = nodes;

%Get the size of the image
[imR, imC] = size(skelMesh);

%Choose a box size for the search
%defaultSearchBoxWidth = 15;
defaultSearchBoxWidth = floor(relocationRadiusScalar*meshSpacingPixels);

%Go through all of the tagged points
try
    parfor i=1:length(nodes)

       %Grab a tagged point 
       currentNodeStruct = nodes(i);

       %Extract the X and Y coordinates
       currentX = floor(currentNodeStruct.nodeX);
       currentY = floor(currentNodeStruct.nodeY);

       %Construct a search grid, protected against going outside the image
       minX = max(1,currentX-defaultSearchBoxWidth);
       maxX = min(imC, currentX+defaultSearchBoxWidth);
       minY = max(1,currentY-defaultSearchBoxWidth);
       maxY = min(imR, currentY+defaultSearchBoxWidth);

       %Places to store information about pixels in the search grid
       distArray = [];
       distArrayPix = [];

       %Calculate the distance between all points in the search grid and the
       %current tagged point's X Y location
       for j = minX:1:maxX
           for k = minY:1:maxY

               %Store the distances in one array that can be sorted later
               distArray = [distArray;  sqrt((currentX-j)^2+(currentY-k)^2)];

               %Store the information for each of those distances' pixels in a
               %separate matrix to access later
               distArrayPix = [distArrayPix; j k skelMesh(k,j)];
           end
       end

       %Sort the distances of the pixels from the current tagged point's X Y
       %location
       [sortedDistArray, origI] = sort(distArray);

       %Iterate through the sorted points
       foundNewSpot = false;
       findNewSpotInd = 1;
       while(~foundNewSpot)

           %If we haven't found a point with a value of zero lets look at the
           %sorted points from closest to furthest
           currentSpotInd = origI(findNewSpotInd);%The original index of the pixel we're looking at
           currentSpot = distArrayPix(currentSpotInd,:);%The original coordinates of the pixel we're looking at

           %If the pixel we're looking at is the right color then save the
           %coordinates and stop the loop
           if(currentSpot(3)==meshColor)
               newX = currentSpot(1);
               newY = currentSpot(2);
               foundNewSpot = true;
           end

           %Increment the search index
           findNewSpotInd = findNewSpotInd+1;
       end

       %Update the node with its new X and Y coordinates
       currentNodeStruct.nodeX = newX;
       currentNodeStruct.nodeY = newY;
       nodes(i) = currentNodeStruct;
    end

    newNodes = nodes;
catch
    %Get a new radis scalar
    prompt = {'The skeleton points could not be shifted successfully, increase the search radius scalar'};
    dlg_title = 'What is the new search radius scalar?';
    num_lines = 1;
    defaultans = {num2str(relocationRadiusScalar)};
    relocationRadiusScalarText = inputdlg(prompt,dlg_title,num_lines, defaultans);
    relocationRadiusScalar = str2num(relocationRadiusScalarText{1});
    [newNodes, oldNodes] = skelMeshPointsModify(skelMesh, nodes, meshColor, meshSpacingPixels, relocationRadiusScalar);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, connectionArray, nodeStructArray)
plotPoints(mask, connectionArray, nodeStructArray);

optList = {'Add Connection', 'Delete Connection', 'Import Into Imaris'};
[option,v] = listdlg('PromptString','Select an Option:',...
                'SelectionMode','single',...
                'ListString',optList);

%Catch a cancel at the main menu
if(isempty(option))
    checkLabels = {'Save Edit Results to Variable Named:','Save Edit Results to Variable Named:'}; 
    varNames = {'newConnectionArray','nodeStructArray'}; 
    items = {connectionArray,nodeStructArray};
    export2wsdlg(checkLabels,varNames,items,...
                'Program Terminated: Save Edit Results to Workspace');
    close all
    error('Program Terminated')
end              
    switch option
        
        %Adding a connection
        case 1
            prompt = {'Pairs:'};
            dlg_title = 'AddConnection';
            num_lines = 4;
            defaultans = {''};
            options.WindowStyle = 'normal';
            answer = inputdlg(prompt,dlg_title,num_lines,defaultans,options);
            
            %If the user clicks cancel jump to main menu
            if(size(answer) == 0)
                [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, connectionArray, nodeStructArray);
            else
            
                %Convert pair text to array of numbers
                [pairs, tf] = str2num(answer{1});

                %If the conversion was possible check that none of the numbers
                %is less than 0 or larger than the number of nodes

                if(tf==1)
                    [~, numNodes] = size(nodeStructArray);
                    inputsOutsideBounds = sum((pairs)<=0)~=0 || sum((pairs)>(numNodes))~=0;
                end

                %Keep allowing the user to edit the list until there is no
                %error (str2num works, even list, no elements outside the
                %number of nodes) also, catch if the user clicks cancel and
                %jump up to the main menu
                initialAns = answer{1};
                runLoop = tf==0 || (mod(numel(pairs),2)~=0) || inputsOutsideBounds;
                cancelState = 0;
                while(runLoop)
                    cancelState = 0;
                    prompt = {'There was an error in your list, please double check:'};
                    dlg_title = 'AddConnection';
                    num_lines = 4;
                    defaultans = {initialAns};
                    options.WindowStyle = 'normal';
                    answer = inputdlg(prompt,dlg_title,num_lines,defaultans,options);
                    
                    if(size(answer) == 0)
                        [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, connectionArray, nodeStructArray);
                        runLoop=0;%user clicked cancel, break the loop
                        cancelState = 1;%user has cancelled, answer is empty
                    else
                        initiailAns = answer{1};
                        
                        %Convert pair text to array of numbers
                        [pairs, tf] = str2num(answer{1});

                        %If the conversion was possible check that none of the numbers
                        %is less than 0 or larger than the number of nodes

                        if(tf==1)
                            [~, numNodes] = size(nodeStructArray);
                            inputsOutsideBounds = sum((pairs)<=0)~=0 || sum((pairs)>(numNodes))~=0;
                        end
                        runLoop = tf==0 || (mod(numel(pairs),2)~=0) || inputsOutsideBounds;
                    end
                end
                
                %don't do anything in here if the user clicked cancel in
                %the error catching state
                if(~cancelState)
                    for i = 1:2:length(pairs)

                        connectionEnds = [pairs(i) pairs(i+1)];
                        connectionHolder = connectionArray;
                        
                        try
                            connectionArray = addConnectionEditing(connectionArray, connectionEnds, nodeStructArray);
                            newConnections = connectionArray;
                            finalNodeStructArray = nodeStructArray;
                        catch
                           newConnections = connectionHolder;
                            finalNodeStructArray = nodeStructArray;
                        end

                    end
                    plotPoints(mask, newConnections, finalNodeStructArray);
                    [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, newConnections, finalNodeStructArray);
                end
                
%                 %Make sure to assign stuff if we're popping the stack to
%                 %finish this script and we cancelled somewhere
%                 if(cancelState)
%                     newConnections = connectionArray;
%                     finalNodeStructArray = finalNodeStructArray;
%                 else
%                     newConnections = newConnections;
%                     finalNodeStructArray = finalNodeStructArray;
%                 end
            end
        case 2
            %Deleting a connection
            prompt = {'Pairs:'};
            dlg_title = 'DeleteConnection';
            num_lines = 4;
            defaultans = {''};
            options.WindowStyle = 'normal';
            answer = inputdlg(prompt,dlg_title,num_lines,defaultans,options);
            
            %If the user clicks cancel jump to main menu
            if(size(answer) == 0)
                [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, connectionArray, nodeStructArray);
            else
                
                %Convert pair text to array of numbers
                [pairs, tf] = str2num(answer{1});

                %If the conversion was possible check that none of the numbers
                %is less than 0 or larger than the number of nodes

                if(tf==1)
                    [~, numNodes] = size(nodeStructArray);
                    inputsOutsideBounds = sum((pairs)<=0)~=0 || sum((pairs)>(numNodes))~=0;
                end

                %Keep allowing the user to edit the list until there is no
                %error (str2num works, even list, no elements outside the
                %number of nodes) also, catch if the user clicks cancel and
                %jump up to the main menu
                initialAns = answer{1};
                runLoop = tf==0 || (mod(numel(pairs),2)~=0) || inputsOutsideBounds;
                cancelState = 0;
                while(runLoop)
                    cancelState = 0;
                    prompt = {'There was an error in your list, please double check:'};
                    dlg_title = 'DeleteConnection';
                    num_lines = 4;
                    defaultans = {answer{1}};
                    options.WindowStyle = 'normal';
                    answer = inputdlg(prompt,dlg_title,num_lines,defaultans,options);

                    if(size(answer) == 0)
                        [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, connectionArray, nodeStructArray);
                        runLoop=0;%user clicked cancel, break the loop
                        cancelState = 1;%user clicked cancel
                    else
                        initiailAns = answer{1};
                        %Convert pair text to array of numbers
                        [pairs, tf] = str2num(answer{1});

                        %If the conversion was possible check that none of the numbers
                        %is less than 0 or larger than the number of nodes

                        if(tf==1)
                            [~, numNodes] = size(nodeStructArray);
                            inputsOutsideBounds = sum((pairs)<=0)~=0 || sum((pairs)>(numNodes))~=0;
                        end
                        runLoop = tf==0 || (mod(numel(pairs),2)~=0) || inputsOutsideBounds;
                    end
                end
                
                %don't do anything in here if the user clicked cancel in
                %the error catching state
                if(~cancelState)
                    for i = 1:2:length(pairs)

                        connectionEnds = [pairs(i) pairs(i+1)];
                        connectionHolder = connectionArray;

                        try
                            connectionArray = deleteConnectionEditing(connectionArray, connectionEnds);
                            newConnections = connectionArray;
                            finalNodeStructArray = nodeStructArray;
                        catch
                            newConnections = connectionHolder;
                            finalNodeStructArray = nodeStructArray;
                        end

                    end

                    plotPoints(mask, newConnections, finalNodeStructArray);
                    [newConnections,finalNodeStructArray] = editConnectionsBetweenSpots(mask, newConnections, finalNodeStructArray);
                end
                
%                 %Make sure to assign stuff if we're popping the stack to
%                 %finish this script and we cancelled somewhere
%                 if(cancelState)
%                     newConnections = connectionArray;
%                     finalNodeStructArray = finalNodeStructArray;
%                 else
%                     newConnections = newConnections;
%                     finalNodeStructArray = finalNodeStructArray;
%                 end
            end
        case 3
            newConnections = connectionArray;
            finalNodeStructArray = nodeStructArray;
    end
           
end

function plotPoints(mask, connectionArray, nodeStructArray)
close
[nr, nc] = size(mask);
scale = 4;
mask = imresize(mask,[nr*scale nc*scale]);
plotPointsFig = figure('visible','off');
imshow(mask,[]);
hold on
locXList = [];
locYList = [];
for k = 1:length(nodeStructArray)
    plotConnections = connectionArray(k).connections;
    currentNode = nodeStructArray(k);
    if(~isempty(plotConnections))
        for j = 1:length(plotConnections)
            attachingNode = nodeStructArray(plotConnections(j));
            %line([currentNode.nodeX attachingNode.nodeX],[currentNode.nodeY attachingNode.nodeY], 'UserData', [k j],'Color', 'c','LineWidth',1.5);
            %h = text(currentNode.nodeX, currentNode.nodeY, num2str(currentNode.nodeID),'FontSize',16,'Color','r');
            %set(h, 'Clipping', 'on');
            locXList = [locXList, [currentNode.nodeX*scale;attachingNode.nodeX*scale], [NaN; NaN]];
            locYList = [locYList, [currentNode.nodeY*scale;attachingNode.nodeY*scale], [NaN; NaN]];
        end
    end
end

%Print the entire line
line(locXList(:),locYList(:),'Color', 'c','LineWidth',1.5);
ax = gca;
ax.Units = 'pixels';
pos = ax.Position;

%Get just the axes
F=getframe(gcf,[pos(1),pos(2),pos(3),pos(4)]);

%Get scaling data to locate text in image
[origHeight, origWidth, n] = size(imresize(F.cdata,[nr*scale nc*scale]));
heightScale = origHeight/nr;
widthScale = origWidth/nc;

%Close the vectorized figure and show it as an image to speed up editing
close(plotPointsFig);
imshow(imresize(F.cdata,[nr*scale nc*scale]));

%Place the text on the image
for k = 1:length(nodeStructArray)
    plotConnections = connectionArray(k).connections;
    currentNode = nodeStructArray(k);
    if(~isempty(plotConnections))
        for j = 1:length(plotConnections)
            h = text(currentNode.nodeX*widthScale, currentNode.nodeY*heightScale, num2str(currentNode.nodeID),'FontSize',16,'Color','r');
            set(h, 'Clipping', 'on');
        end
    end
end
end


function newConnections = addConnectionEditing(connectionArray, connectionEnds, nodeStructArray)
%Get the connections for each node
connections1 = connectionArray(connectionEnds(1)).connections;
connections2 = connectionArray(connectionEnds(2)).connections;

%Append the new connection to the connections
connections1 = [connections1 connectionEnds(2)];
connections2 = [connections2 connectionEnds(1)];

%Set the stored value to the new value
connectionArray(connectionEnds(1)).connections = connections1;
connectionArray(connectionEnds(2)).connections = connections2;

newConnections = connectionArray;
end

function newConnections = deleteConnectionEditing(connectionArray, connectionEnds)

%Get the connections for each node
connectionArray(connectionEnds(1));
connectionArray(connectionEnds(2));
connections1 = connectionArray(connectionEnds(1)).connections;
connections2 = connectionArray(connectionEnds(2)).connections;

%Find where the connection to be removed is
firstInd = find(connections1==connectionEnds(2));
secondInd = find(connections2==connectionEnds(1));

numConnections = length(firstInd);

%Generate the string for the new connectionarray without the connection to
%be removed
for i = 1:numConnections  
    connections1 = connectionArray(connectionEnds(1)).connections;
    connections2 = connectionArray(connectionEnds(2)).connections;
    
    firstInd = find(connections1==connectionEnds(2));
    secondInd = find(connections2==connectionEnds(1));
    
    firstStr = sprintf('[connections1(1:%d) connections1(%d:%d)]',firstInd(1)-1, firstInd(1)+1, length(connections1));
    secondStr = sprintf('[connections2(1:%d) connections2(%d:%d)]',secondInd(1)-1, secondInd(1)+1, length(connections2));
    
    %Set the stored connections to the evaluated strings
    eval(firstStr);
    eval(secondStr);
    
    connectionArray(connectionEnds(1)).connections = eval(firstStr);
    connectionArray(connectionEnds(2)).connections = eval(secondStr);

    newConnections = connectionArray;
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [lineMatrix,numLineArray] = generateFilamentMatrix(connectionArray, referenceNodeStructArray, trackingInformation, numPointsInLines)

%Determine the number of time points and unique trackIDs
numTimePoints = length(unique(trackingInformation(:,4)));
numTrackID = length(unique(trackingInformation(:,5)));

%Preallocate storage of the lines
lineMatrix = cell(1,numTimePoints);
numLineArray = zeros(1,numTimePoints);

%Generate the lines for each time point
for i = 1:numTimePoints
   [lineData, numLines] = generateFilaments(connectionArray, referenceNodeStructArray, trackingInformation(floor(trackingInformation(:,4)) == i-1,:),numPointsInLines);
   numLineArray(1,i) = numLines;
   lineMatrix{1,i} = lineData;
end


end

function [lineData, numLines] = generateFilaments(connectionArray, referenceNodeStructArray, trackingInformation, numPointsInLines)
%global attachingNodeXYZ attachingNode
%Use the connection array generated from the first time point to construct
%the lines for later time points. We know the trackID for each node and how
%all the nodes are connected to each other. In later time points the
%trackID values are not sorted but you can search the tracking data by using 
%the trackIDs from the reference connections
lineData = cell(length(referenceNodeStructArray),1);
numLines = 0;
    for k = 1:length(referenceNodeStructArray)
        plotConnections = connectionArray(k).connections;
        currentNode = referenceNodeStructArray(k);
        
        %use the known connection pattern to pull the information from the
        %tracking data
        currentNodeXYZ = trackingInformation(trackingInformation(:,5)==currentNode.trackID,:);

        if(~isempty(plotConnections))
            for j = 1:length(plotConnections)
                attachingNode = referenceNodeStructArray(plotConnections(j));
                    
                %use the known connection pattern to pull the information from the
                %tracking data
                attachingNodeXYZ = trackingInformation(trackingInformation(:,5)==attachingNode.trackID,:);
                
                xLinspace = linspace(currentNodeXYZ(:,1), attachingNodeXYZ(:,1),numPointsInLines);
                yLinspace = linspace(currentNodeXYZ(:,2), attachingNodeXYZ(:,2),numPointsInLines);
                zLinspace = linspace(currentNodeXYZ(:,3), attachingNodeXYZ(:,3),numPointsInLines);
                
                currentLine.X = xLinspace;
                currentLine.Y = yLinspace;
                currentLine.Z = zLinspace;
                
                lineData{k} = [lineData{k} currentLine]; 
                
                %delete the connection so you don't repeat it
                connectionArray = deleteConnectionAfterFilamentAdding(connectionArray, [currentNode.nodeID attachingNode.nodeID]);
                numLines = numLines+1;
            end
        end
    end

end

function newConnections = deleteConnectionAfterFilamentAdding(connectionArray, connectionEnds)

%Get the connections for each node
connections1 = connectionArray(connectionEnds(1)).connections;
connections2 = connectionArray(connectionEnds(2)).connections;

%Find where the connection to be removed is
firstInd = find(connections1==connectionEnds(2));
secondInd = find(connections2==connectionEnds(1));

numConnections = length(firstInd);

%Generate the string for the new connectionarray without the connection to
%be removed
for i = 1:numConnections  
    connections1 = connectionArray(connectionEnds(1)).connections;
    connections2 = connectionArray(connectionEnds(2)).connections;
    
    firstInd = find(connections1==connectionEnds(2));
    secondInd = find(connections2==connectionEnds(1));
    
    firstStr = sprintf('[connections1(1:%d) connections1(%d:%d)]',firstInd(1)-1, firstInd(1)+1, length(connections1));
    secondStr = sprintf('[connections2(1:%d) connections2(%d:%d)]',secondInd(1)-1, secondInd(1)+1, length(connections2));

    %Set the stored connections to the evaluated strings
    
    eval(firstStr);
    eval(secondStr);
    
    connectionArray(connectionEnds(1)).connections = eval(firstStr);
    connectionArray(connectionEnds(2)).connections = eval(secondStr);

    newConnections = connectionArray;
end

newConnections = connectionArray;

end

function imarisInput = generateImarisFilamentInput(lineMatrix, numPointsInLines)

numTimePoints = length(lineMatrix);
numPoints = length(lineMatrix{1});

totalNumLines = 0;
testTimePoint = lineMatrix{1};
for i = 1:length(testTimePoint)
   totalNumLines = totalNumLines + length(testTimePoint{i}); 
end
totalLineMatrix = [];

%Store all the lines in the format IMARIS expects
for i = 1:numPoints
    numConnections = length(lineMatrix{1}{i});
    for j = 1:numConnections
        currentLineMatrix = [];

        for k = 1:numTimePoints
            currentPointConnections = lineMatrix{k}{i};
            if(length(currentPointConnections)>0)
                currentLine = currentPointConnections(j);
                currentLineMatrix = [currentLineMatrix; currentLine.X' currentLine.Y' currentLine.Z'];
            end
            
        end
        %pause
        totalLineMatrix = [totalLineMatrix; currentLineMatrix];
    end
end

imarisInput = totalLineMatrix;


end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function AddCustomFilaments(aImarisApplicationID, imarisInput, numLineArray, numPointsInLine, numTimePoints)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For each line segment we will need to add a filament as follows:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% vFactory = vImarisApplication.GetFactory;
        % vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);

        % aFilament = vImarisApplication.GetFactory.ToFilaments(vImarisApplication.GetSurpassSelection);
        % aPositionsXYZ = aFilament.GetPositionsXYZ;

        %% The Filament object is assumed to be selected in the Surpass Scene tree.
        % vFilament = vImarisApplication.GetFactory.ToFilaments(vImarisApplication.GetSurpassSelection);
        % vFilamentList = vFilament.GetFilamentsList(1);
        % vPositionsXYZ = vFilamentList.mPositionsXYZ;
        % vIds = vFilamentList.mIdPerFilament;

vNewFilament = vImarisApplication.GetFactory.CreateFilaments;

aNumFilaments = min(numLineArray); % This will be ann inputed value 
aNumTimePoints = numTimePoints; % We will know this value when running normally, or user input
aFilamentRadius = 0.6;
aNumPointsPerFilament = numPointsInLine; % This value is to be determined and set by the user 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% aPositions is a matrix [X,Y,Z] 'single' for each point position along the
% legth of the filament. Each filament if made up of a series of positions 
% connected by edges that define the length of the filament segment. For 
% the purpose of this exstention, each filament will contain 20 position 
% evenly distributed along the length of the path between two nodes. 


%Importing Positions from results of Node connections identification
aDataImport = imarisInput;

aImarisImport = aDataImport;

aPositions_F = single(aImarisImport);
 
            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% An array containing number of key points per filament   
    
aNumPointsForEachFilament = zeros(1,aNumFilaments*aNumTimePoints,'int32');
        for i = 1:aNumFilaments*aNumTimePoints
            aNumPointsForEachFilament(1,i) = aNumPointsPerFilament;
        end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Each filament contains 0-19 edges that define the points relative 
% location and relationship to each other. Each the position of each node 
% should be defined by edge 0 and 19 with units of 'int32'. 

aEdgesBase = zeros(aNumPointsPerFilament-1,2,'int32');
        for i = 1:aNumPointsPerFilament-1
            aEdgesBase(i,1) = i-1;
            aEdgesBase(i,2) = i;
        end
 
aEdges_F = [];
        for i = 1:aNumFilaments*aNumTimePoints 
            aEdges_F = [aEdges_F; aEdgesBase];
        end
        
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% An array containing number of key points per filament   
    
aNumEdgesPerFilament = zeros(1,aNumFilaments*aNumTimePoints,'int32');
        for i = 1:aNumFilaments*aNumTimePoints
            aNumEdgesPerFilament(1,i) = aNumPointsPerFilament-1;
        end
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The radi is determined by the users as the filament diameter to render in
% 3D space. This value can be changed as desiered, and altered later in
% Imaris. This needs to be called as a 'single' matrix. 

aRadiiBase = zeros(aNumPointsPerFilament,1,'single');
        for i = 1:aNumPointsPerFilament
            aRadiiBase(i) = aFilamentRadius;
        end 

aRadii_F = [];
        for i = 1:aNumFilaments*aNumTimePoints 
            aRadii_F = [aRadii_F; aRadiiBase];
        end
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The type determiens the identity of the FilamentType as either a
% dendrite=0, or a spine=1. Future code can ask for a user input here to
% determine if the user is creating dendrite segments or spines. This must
% be a martix of type 'int32'. 

aTypeBase = zeros(aNumPointsPerFilament,1,'int32');
    FilamentType = 0;
        for i = 1:aNumPointsPerFilament
            aTypeBase(i) = FilamentType;
        end
        
aType_F = [];
        for i = 1:aNumFilaments*aNumTimePoints 
            aType_F = [aType_F; aTypeBase];
        end
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This specifies the time points for which this filament is present as a
% normal 1xN double matrix. As the filament is added to more time points,
% this matrix increases from 0 to time X, with each time point added to a
% new column. When more than one filament is present in a time point, time
% is indicated for all instances of the first filamnet, then the
% second and so on.  

BaseIndex = zeros(1,aNumTimePoints);
    for i = 1:aNumTimePoints
        BaseIndex(1,i) = i-1;
    end
    
aTimeIndex = [];
    for i = 1:aNumFilaments
        aTimeIndex = [aTimeIndex BaseIndex];
    end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is setting the color of the filament. I am not sure what the values
% mean, but it seems to be a common value. Standard 1x1 double. 

vRGBA = 13421772;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adding the tracking information as FilTrackEdges. This is similar to the
% format of aEdges but it is for defining the Filaments in time that are
% conencted to each other. This is a M x N matrix of int32. 

BaseTimeIndex = zeros(aNumTimePoints-1,2);
    for i = 1:aNumTimePoints-1
        BaseTimeIndex(i,1) = i-1;
        BaseTimeIndex(i,2) = i;
    end
    
aFilTrackEdges = BaseTimeIndex;
NextTimeIndex = BaseTimeIndex;
    for i = 1:aNumFilaments-1
        NextTimeIndex = NextTimeIndex + aNumTimePoints;
        aFilTrackEdges = cat(1,aFilTrackEdges, NextTimeIndex);
    end
    
 aFilTrackEdges = int32(aFilTrackEdges);


    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adding the new filament to the Imaris scene. 
vNewFilament.AddFilamentsList(aPositions_F, aNumPointsForEachFilament, aRadii_F, aType_F, aEdges_F, aNumEdgesPerFilament, aTimeIndex);
vNewFilament.SetName('ImportedFilament');
vNewFilament.SetFilamentTrackEdges(aFilTrackEdges);
vNewFilament.SetColorRGBA(vRGBA);
vImarisApplication.GetSurpassScene.AddChild(vNewFilament, -1);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function ImportFilamentStrainStatistic(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Filaments Objects and data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);% Need to have the filaments objected selected in surpass view
vFilamentsIds = vFilaments.GetIds;
vFilamentsTrackIds = vFilaments.GetFilamentTrackIds;
vFilamentsTrackEdges = vFilaments.GetFilamentTrackEdges;

% Counting number of cells
vCountFilaments = vFilaments.GetNumberOfFilaments;

% Getting the Statistics 
    vAllStatistics = vFilaments.GetStatistics;
    vNames       = cell(vAllStatistics.mNames);
    vValues      = vAllStatistics.mValues;
    vUnits       = cell(vAllStatistics.mUnits); % not used
    vFactors     = cell(vAllStatistics.mFactors);
    vFactorNames = cellstr(char(vAllStatistics.mFactorNames));
    vIds         = vAllStatistics.mIds;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the Name of the Similarity Statistics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vPrompt      = 'Enter the name of the strain statistic (Strain prefix will be added)';
vDlgTitle    = 'Strain Statistics Name';
vNumOfLines  = 1;
vDefaultName = {''};
vStrainStatisticsName = char(inputdlg(vPrompt,vDlgTitle,vNumOfLines,vDefaultName));
if isempty(vStrainStatisticsName) 
    return 
end


% Extracting the Dendrite Ids and the Ids to build Statistic 

vStatisticDendriteLength = find(strcmp('Dendrite Length', vNames)); 
vExtractedDendriteLength = double(vValues(vStatisticDendriteLength));      
    
vStatisticObjectId = find(strcmp('Filament Time Index', vNames)); 
vExtractedObjectIds = double(vIds(vStatisticObjectId));   
    
vStatisticId = find(strcmp('Dendrite Time Index', vNames));      
vExtractedDendriteIds = double(vIds(vStatisticId));   
    
vStatisticTime = find(strcmp('Dendrite Time Index', vNames)); 
vExtractedTime = double(vValues(vStatisticTime));   

FilamentLengthMatrix = [vExtractedDendriteLength, vExtractedTime, vExtractedDendriteIds, vExtractedObjectIds];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Importing an Excel Sheet with the tracked data for cells with the data
% sorted by TrackID then by Time point 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SortedStrainMatrix =  MeshStrainLength(FilamentLengthMatrix,vFilamentsTrackEdges); % These are independent functions needed to accompany this script 
[row,column] = size(SortedStrainMatrix);% Calculating the size [rows,columns] of data matrix

%%%%%%%%%%%%%%%%%%%%%%%%%
% Set New Track Statistic
%%%%%%%%%%%%%%%%%%%%%%%%%
vIndT=1:length(SortedStrainMatrix(:,3));% Total number of Track IDs
vIdsT= FilamentLengthMatrix(:,3);% Extracting the Dendrite ID #'s
vUnitsT(vIndT) = {'strain'};% Setting the units 

% Creating vFactors string array
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling the Imaris object description 
% Loop to build the additional rows in the string for vFactorT 
% that would contain Channel information,collection, and time. These 
% categories need to match the original vFactors that are found when 
% exporting all statistics.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFactorsT(vIndT) = {'Dendrite'};% Row 1 of vFactorsT is the Object 'Cell' for all 
    for i = 1:row
        vFactorsT(2,i) = {''};% Blank rows 
        vFactorsT(3,i) = {num2str(SortedStrainMatrix(i,2))};% Extracting the time points from SortedStrainMatrix and making a string 
        vFactorsT(4,i) = {''};% Blank rows (This stands for 0 since it is of type dendrite)
    end

% Creating Remaining string arrays
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vNamesT(vIndT) = {['Strain',' ',vStrainStatisticsName]};%Statistic name that will appear in the dropdown menu
vFactorNamesT = {'Category','Collection','Time','Type',};
vStrainStatistic = SortedStrainMatrix(:,1)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Exporting the New Statistic to Imaris
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vFilaments.AddStatistics(vNamesT, vStrainStatistic, vUnitsT, vFactorsT, vFactorNamesT, vIdsT); 

end

% function [normalized] = MeshStrainLength(FilamentLengthMatrix)
% 
% [sorted,sortOrder] = sort(FilamentLengthMatrix(:,2));%sort the matrix by ID
% sortedCellLengthMatrix = FilamentLengthMatrix(sortOrder,:);%sort the matrix by ID
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Get the Number of Time Points Per Filament
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% vNumTimePoints = max(FilamentLengthMatrix(:,2));
% 
% vLengthNormalized = []; % Creating an empty matrix to append the normalized data to
% 
% for i = 1: vNumTimePoints: (length(sortedCellLengthMatrix(:,1)))
%     currentArray = sortedCellLengthMatrix(i:i+(vNumTimePoints-1),:); % Looking at all rows, column 3 of cellAreaMatrix in steps of Number of Time Points
%     currentArray(:,1) = currentArray(:,1)./currentArray(1,1); % All rows first column of current Array is equal to all rows first column of current Array divided by the first row first column of currentArray
%     vLengthNormalized = [vLengthNormalized;currentArray]; % creating normalized as the final data matrix and appending each itteration of the loop from currentArray to the normalized data matrix
% end
% 
% normalized = vLengthNormalized; 
% 
% end

function [normalized] = MeshStrainLength(FilamentLengthMatrix,vFilamentsTrackEdges)

%assign trackID to appropriate locations in the matrix
%subtract 1 to account for fencepost problem (edges are one fewer in number
%than points). This is why there are -2s in certain places
vNumFilaments = numel(find(FilamentLengthMatrix(:,2)==1));
vNumTimePoints = max(FilamentLengthMatrix(:,2));
firstObjectID = FilamentLengthMatrix(1,4);
normalized = FilamentLengthMatrix;

%iterate through the filaments
for i = 1:vNumFilaments
    vFilamentsTrackEdgesShifted = vFilamentsTrackEdges(((i-1)*vNumTimePoints)+2-i:(i)*vNumTimePoints-i,1);%Get the edges for all time points for this filament  
    vFilamentsTrackEdgesShifted =[vFilamentsTrackEdgesShifted; vFilamentsTrackEdges((i)*vNumTimePoints-i,2)];%Append the final object ID for the filament
    
    setFirstObjectID=vFilamentsTrackEdgesShifted(1)+firstObjectID;%get the first object ID for this set of filaments
    firstObjectRowInFilLengthMatrix = FilamentLengthMatrix(find(FilamentLengthMatrix(:,4)==setFirstObjectID),:);%get the row corresponding to the first object ID
    initialLength = firstObjectRowInFilLengthMatrix(1);%get the initial length for this set of filaments
    %iterate through this set of filaments
    for j = 1:length(vFilamentsTrackEdgesShifted)
        currentObjectID=vFilamentsTrackEdgesShifted(j)+firstObjectID
        currentRowNumber = find(FilamentLengthMatrix(:,4)==currentObjectID)
        currentRow = FilamentLengthMatrix(currentRowNumber,:)
        currentStrain = (currentRow(1)-initialLength)/initialLength;
        normalized(currentRowNumber,1) = currentStrain;
    end
end
% [sorted,sortOrder] = sort(FilamentLengthMatrix(:,3));%sort the matrix by ID
% sortedCellLengthMatrix = FilamentLengthMatrix(sortOrder,:);%sort the matrix by ID
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % Get the Number of Time Points Per Filament
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% vNumTimePoints = max(FilamentLengthMatrix(:,2));
% vNumFilaments = numel(find(FilamentLengthMatrix(:,2)==1));
% 
% vLengthNormalized = zeros(vNumTimePoints,1); % Creating an empty matrix to append the normalized data to
% 
% ReferenceArray = sortedCellLengthMatrix(1:1+(vNumFilaments-1),1);
% 
% for j = 1:vNumFilaments
%     for i = j: vNumFilaments: (length(sortedCellLengthMatrix(:,1)))
%         currentNum = sortedCellLengthMatrix(i,1); % Looking at all rows, column 3 of cellAreaMatrix in steps of Number of Time Points
%         currentNumNormalized = currentNum/ReferenceArray(j);
%         vLengthNormalized(i) = currentNumNormalized; % creating normalized as the final data matrix and appending each itteration of the loop from currentArray to the normalized data matrix
%     end
% end
% 
% normalized = [vLengthNormalized sortedCellLengthMatrix(:,2:end)]; 

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


