%This function plots the dilation calculated in imaris as a grid of time
%series plots. sortedDilationData is the data exported from Imaris with the
%four following columns [dilation, time point, trackID, ID] and needs to be
%sorted by timepoint. The trackIDTopLeftRowWise is a matrix of track IDs
%in positions matching your NMBS 
function plotImarisDilation(sortedDilationData, trackIDTopLeftRowWise)

%Program Settings
numRows = 5;
numCols = 5;
numTimePoints = 20;
plotFileType ='jpg';
figureWidth = 2000;

%Save Time Series Plots for Exx
[timeSeriesDilationFig,cbarDilationFig] = timeSeriesPlot(trackIDTopLeftRowWise, sortedDilationData, numRows, numCols, numTimePoints);

%Get the user's desired location for the plots
filePath = uigetdir(matlabroot,'Please Select the Plot Save Location');

%Set size of figure for saving
timeSeriesDilationFig.Position=[0 0 figureWidth figureWidth];
cbarDilationFig.Position=[0 0 figureWidth figureWidth];
newFolder = fullfile(filePath, 'TimeSeriesDilation');
[status, msg, msgID]=mkdir(newFolder);
filePathToSave = fullfile(newFolder, 'TimeSeriesDilation');
saveas(timeSeriesDilationFig,filePathToSave,plotFileType);
filePathToSave = fullfile(newFolder, 'TimeSeriesDilationColorBar');
saveas(cbarDilationFig,filePathToSave,plotFileType);
end

%This function plots the time series data for a specific dilation for each of
%the square elements of the NMBS in subplots 
function [timeSeriesFig,cbarFig] = timeSeriesPlot(trackIDTopLeftRowWise, dilationData, numRows, numCols, numTimePoints)

%Generate the figure and pull in the colormap
timeSeriesFig = figure;
colormap(getPhase);
axisScaleMax = 0;

%Iterate through the NMBS square elements
for i = 1:numRows*numCols
    
    transpTrackID = trackIDTopLeftRowWise';
    
    %Pull out the strain data for this NMBS square element
    trackIDDilationData = dilationData(find(dilationData(:,3)==transpTrackID(i)),:);
    currentDilationData = trackIDDilationData(:,1);
    currentAxisScaleMax = max(abs(currentDilationData-1));
    if(currentAxisScaleMax>axisScaleMax)
        axisScaleMax = currentAxisScaleMax;
    end    
    
    %Generate a subplot in the correct location
    subplot(numRows,numCols,i);

    
    %Plot the strain data with a zero line, markers, and a connecting line
    plot(linspace(0,numTimePoints,numTimePoints),currentDilationData,'LineWidth',1,'Color','k')
    hold on
    scatter(linspace(0,numTimePoints,numTimePoints),currentDilationData,600,currentDilationData,'filled')
    plot(linspace(0,numTimePoints,numTimePoints),ones(1,numTimePoints),'LineStyle','--','Color','k','LineWidth',2)
    
    %Format the plot 
    box off
    ax = gca;
    ax.LineWidth = 4;
    
    %Option to not have x and y labels
    ax.YTick=[];
    ax.XTick=[];
end

%Scale all the axes the same
for i = 1:numRows*numCols
    subplot(numRows,numCols,i);
    axis([0, numTimePoints, 1-1.1*axisScaleMax, 1+1.1*axisScaleMax]);
    cbar = colorbar;
    cbar.Limits = [1-1.1*axisScaleMax 1+1.1*axisScaleMax];
    caxis([1-1.1*axisScaleMax 1+1.1*axisScaleMax]);
    colorbar('off');
end

%Hide the time series figure
timeSeriesFig.Visible = 'off';

%Make a colorbar figure
cbarFig = figure;
colormap(getPhase);
cbar = colorbar;
cbar.Limits = [1-1.1*axisScaleMax 1+1.1*axisScaleMax];
caxis([1-1.1*axisScaleMax 1+1.1*axisScaleMax]);

%Hide the colorbar figure
cbarFig.Visible = 'off';
end

%Generates a custom phase colormap
function custcm = getPhase
custcm = [[0,0,0.988235294117647;0,0,0.988235294117647;0,0,0.988235294117647;0.0156862745098039,0.0156862745098039,0.988235294117647;0.0156862745098039,0.0156862745098039,0.988235294117647;0.0156862745098039,0.0156862745098039,0.988235294117647;0.0313725490196078,0.0313725490196078,0.988235294117647;0.0313725490196078,0.0313725490196078,0.988235294117647;0.0470588235294118,0.0470588235294118,0.988235294117647;0.0470588235294118,0.0470588235294118,0.988235294117647;0.0470588235294118,0.0470588235294118,0.972549019607843;0.0627450980392157,0.0627450980392157,0.972549019607843;0.0627450980392157,0.0627450980392157,0.972549019607843;0.0784313725490196,0.0784313725490196,0.972549019607843;0.0784313725490196,0.0784313725490196,0.972549019607843;0.0784313725490196,0.0784313725490196,0.972549019607843;0.0941176470588235,0.0941176470588235,0.972549019607843;0.0941176470588235,0.0941176470588235,0.972549019607843;0.109803921568627,0.109803921568627,0.972549019607843;0.109803921568627,0.109803921568627,0.972549019607843;0.109803921568627,0.109803921568627,0.956862745098039;0.125490196078431,0.125490196078431,0.956862745098039;0.125490196078431,0.125490196078431,0.956862745098039;0.141176470588235,0.141176470588235,0.956862745098039;0.141176470588235,0.141176470588235,0.956862745098039;0.141176470588235,0.141176470588235,0.956862745098039;0.156862745098039,0.156862745098039,0.956862745098039;0.156862745098039,0.156862745098039,0.956862745098039;0.172549019607843,0.172549019607843,0.956862745098039;0.172549019607843,0.172549019607843,0.956862745098039;0.172549019607843,0.172549019607843,0.941176470588235;0.188235294117647,0.188235294117647,0.941176470588235;0.188235294117647,0.188235294117647,0.941176470588235;0.203921568627451,0.203921568627451,0.941176470588235;0.203921568627451,0.203921568627451,0.941176470588235;0.203921568627451,0.203921568627451,0.941176470588235;0.219607843137255,0.219607843137255,0.941176470588235;0.219607843137255,0.219607843137255,0.941176470588235;0.235294117647059,0.235294117647059,0.941176470588235;0.235294117647059,0.235294117647059,0.925490196078431;0.235294117647059,0.235294117647059,0.925490196078431;0.250980392156863,0.250980392156863,0.925490196078431;0.250980392156863,0.250980392156863,0.925490196078431;0.266666666666667,0.266666666666667,0.925490196078431;0.266666666666667,0.266666666666667,0.925490196078431;0.266666666666667,0.266666666666667,0.925490196078431;0.282352941176471,0.282352941176471,0.925490196078431;0.282352941176471,0.282352941176471,0.925490196078431;0.298039215686275,0.298039215686275,0.925490196078431;0.298039215686275,0.298039215686275,0.909803921568627;0.298039215686275,0.298039215686275,0.909803921568627;0.313725490196078,0.313725490196078,0.909803921568627;0.313725490196078,0.313725490196078,0.909803921568627;0.329411764705882,0.329411764705882,0.909803921568627;0.329411764705882,0.329411764705882,0.909803921568627;0.329411764705882,0.329411764705882,0.909803921568627;0.345098039215686,0.345098039215686,0.909803921568627;0.345098039215686,0.345098039215686,0.909803921568627;0.360784313725490,0.360784313725490,0.909803921568627;0.360784313725490,0.360784313725490,0.894117647058824;0.360784313725490,0.360784313725490,0.894117647058824;0.376470588235294,0.376470588235294,0.894117647058824;0.376470588235294,0.376470588235294,0.894117647058824;0.376470588235294,0.376470588235294,0.894117647058824;0.392156862745098,0.392156862745098,0.894117647058824;0.392156862745098,0.392156862745098,0.894117647058824;0.407843137254902,0.407843137254902,0.894117647058824;0.407843137254902,0.407843137254902,0.894117647058824;0.407843137254902,0.407843137254902,0.878431372549020;0.423529411764706,0.423529411764706,0.878431372549020;0.423529411764706,0.423529411764706,0.878431372549020;0.439215686274510,0.439215686274510,0.878431372549020;0.439215686274510,0.439215686274510,0.878431372549020;0.439215686274510,0.439215686274510,0.878431372549020;0.454901960784314,0.454901960784314,0.878431372549020;0.454901960784314,0.454901960784314,0.878431372549020;0.470588235294118,0.470588235294118,0.878431372549020;0.470588235294118,0.470588235294118,0.878431372549020;0.470588235294118,0.470588235294118,0.862745098039216;0.486274509803922,0.486274509803922,0.862745098039216;0.486274509803922,0.486274509803922,0.862745098039216;0.501960784313726,0.501960784313726,0.862745098039216;0.501960784313726,0.501960784313726,0.862745098039216;0.501960784313726,0.501960784313726,0.862745098039216;0.517647058823530,0.517647058823530,0.862745098039216;0.517647058823530,0.517647058823530,0.862745098039216;0.533333333333333,0.533333333333333,0.862745098039216;0.533333333333333,0.533333333333333,0.862745098039216;0.533333333333333,0.533333333333333,0.847058823529412;0.549019607843137,0.549019607843137,0.847058823529412;0.549019607843137,0.549019607843137,0.847058823529412;0.564705882352941,0.564705882352941,0.847058823529412;0.564705882352941,0.564705882352941,0.847058823529412;0.564705882352941,0.564705882352941,0.847058823529412;0.580392156862745,0.580392156862745,0.847058823529412;0.580392156862745,0.580392156862745,0.847058823529412;0.596078431372549,0.596078431372549,0.847058823529412;0.596078431372549,0.596078431372549,0.831372549019608;0.596078431372549,0.596078431372549,0.831372549019608;0.611764705882353,0.611764705882353,0.831372549019608;0.611764705882353,0.611764705882353,0.831372549019608;0.627450980392157,0.627450980392157,0.831372549019608;0.627450980392157,0.627450980392157,0.831372549019608;0.627450980392157,0.627450980392157,0.831372549019608;0.643137254901961,0.643137254901961,0.831372549019608;0.643137254901961,0.643137254901961,0.831372549019608;0.658823529411765,0.658823529411765,0.831372549019608;0.658823529411765,0.658823529411765,0.815686274509804;0.658823529411765,0.658823529411765,0.815686274509804;0.674509803921569,0.674509803921569,0.815686274509804;0.674509803921569,0.674509803921569,0.815686274509804;0.690196078431373,0.690196078431373,0.815686274509804;0.690196078431373,0.690196078431373,0.815686274509804;0.690196078431373,0.690196078431373,0.815686274509804;0.705882352941177,0.705882352941177,0.815686274509804;0.705882352941177,0.705882352941177,0.815686274509804;0.721568627450980,0.721568627450980,0.815686274509804;0.721568627450980,0.721568627450980,0.800000000000000;0.721568627450980,0.721568627450980,0.800000000000000;0.737254901960784,0.737254901960784,0.800000000000000;0.737254901960784,0.737254901960784,0.800000000000000;0.752941176470588,0.752941176470588,0.800000000000000;0.752941176470588,0.752941176470588,0.800000000000000;0.752941176470588,0.752941176470588,0.800000000000000;0.768627450980392,0.768627450980392,0.800000000000000;0.768627450980392,0.768627450980392,0.800000000000000;0.784313725490196,0.784313725490196,0.784313725490196;0.784313725490196,0.784313725490196,0.784313725490196;0.784313725490196,0.784313725490196,0.784313725490196;0.784313725490196,0.768627450980392,0.768627450980392;0.784313725490196,0.768627450980392,0.768627450980392;0.784313725490196,0.768627450980392,0.768627450980392;0.784313725490196,0.752941176470588,0.752941176470588;0.784313725490196,0.752941176470588,0.752941176470588;0.784313725490196,0.737254901960784,0.737254901960784;0.784313725490196,0.737254901960784,0.737254901960784;0.800000000000000,0.737254901960784,0.737254901960784;0.800000000000000,0.721568627450980,0.721568627450980;0.800000000000000,0.721568627450980,0.721568627450980;0.800000000000000,0.705882352941177,0.705882352941177;0.800000000000000,0.705882352941177,0.705882352941177;0.800000000000000,0.705882352941177,0.705882352941177;0.800000000000000,0.690196078431373,0.690196078431373;0.800000000000000,0.690196078431373,0.690196078431373;0.800000000000000,0.674509803921569,0.674509803921569;0.800000000000000,0.674509803921569,0.674509803921569;0.815686274509804,0.674509803921569,0.674509803921569;0.815686274509804,0.658823529411765,0.658823529411765;0.815686274509804,0.658823529411765,0.658823529411765;0.815686274509804,0.658823529411765,0.658823529411765;0.815686274509804,0.643137254901961,0.643137254901961;0.815686274509804,0.643137254901961,0.643137254901961;0.815686274509804,0.627450980392157,0.627450980392157;0.815686274509804,0.627450980392157,0.627450980392157;0.815686274509804,0.627450980392157,0.627450980392157;0.815686274509804,0.611764705882353,0.611764705882353;0.831372549019608,0.611764705882353,0.611764705882353;0.831372549019608,0.596078431372549,0.596078431372549;0.831372549019608,0.596078431372549,0.596078431372549;0.831372549019608,0.596078431372549,0.596078431372549;0.831372549019608,0.580392156862745,0.580392156862745;0.831372549019608,0.580392156862745,0.580392156862745;0.831372549019608,0.564705882352941,0.564705882352941;0.831372549019608,0.564705882352941,0.564705882352941;0.831372549019608,0.564705882352941,0.564705882352941;0.831372549019608,0.549019607843137,0.549019607843137;0.847058823529412,0.549019607843137,0.549019607843137;0.847058823529412,0.533333333333333,0.533333333333333;0.847058823529412,0.533333333333333,0.533333333333333;0.847058823529412,0.533333333333333,0.533333333333333;0.847058823529412,0.517647058823530,0.517647058823530;0.847058823529412,0.517647058823530,0.517647058823530;0.847058823529412,0.517647058823530,0.517647058823530;0.847058823529412,0.501960784313726,0.501960784313726;0.847058823529412,0.501960784313726,0.501960784313726;0.847058823529412,0.486274509803922,0.486274509803922;0.862745098039216,0.486274509803922,0.486274509803922;0.862745098039216,0.486274509803922,0.486274509803922;0.862745098039216,0.470588235294118,0.470588235294118;0.862745098039216,0.470588235294118,0.470588235294118;0.862745098039216,0.454901960784314,0.454901960784314;0.862745098039216,0.454901960784314,0.454901960784314;0.862745098039216,0.454901960784314,0.454901960784314;0.862745098039216,0.439215686274510,0.439215686274510;0.862745098039216,0.439215686274510,0.439215686274510;0.862745098039216,0.423529411764706,0.423529411764706;0.878431372549020,0.423529411764706,0.423529411764706;0.878431372549020,0.423529411764706,0.423529411764706;0.878431372549020,0.407843137254902,0.407843137254902;0.878431372549020,0.407843137254902,0.407843137254902;0.878431372549020,0.392156862745098,0.392156862745098;0.878431372549020,0.392156862745098,0.392156862745098;0.878431372549020,0.392156862745098,0.392156862745098;0.878431372549020,0.376470588235294,0.376470588235294;0.878431372549020,0.376470588235294,0.376470588235294;0.894117647058824,0.376470588235294,0.376470588235294;0.894117647058824,0.360784313725490,0.360784313725490;0.894117647058824,0.360784313725490,0.360784313725490;0.894117647058824,0.345098039215686,0.345098039215686;0.894117647058824,0.345098039215686,0.345098039215686;0.894117647058824,0.345098039215686,0.345098039215686;0.894117647058824,0.329411764705882,0.329411764705882;0.894117647058824,0.329411764705882,0.329411764705882;0.894117647058824,0.313725490196078,0.313725490196078;0.894117647058824,0.313725490196078,0.313725490196078;0.909803921568627,0.313725490196078,0.313725490196078;0.909803921568627,0.298039215686275,0.298039215686275;0.909803921568627,0.298039215686275,0.298039215686275;0.909803921568627,0.282352941176471,0.282352941176471;0.909803921568627,0.282352941176471,0.282352941176471;0.909803921568627,0.282352941176471,0.282352941176471;0.909803921568627,0.266666666666667,0.266666666666667;0.909803921568627,0.266666666666667,0.266666666666667;0.909803921568627,0.266666666666667,0.266666666666667;0.909803921568627,0.250980392156863,0.250980392156863;0.925490196078431,0.250980392156863,0.250980392156863;0.925490196078431,0.235294117647059,0.235294117647059;0.925490196078431,0.235294117647059,0.235294117647059;0.925490196078431,0.235294117647059,0.235294117647059;0.925490196078431,0.219607843137255,0.219607843137255;0.925490196078431,0.219607843137255,0.219607843137255;0.925490196078431,0.203921568627451,0.203921568627451;0.925490196078431,0.203921568627451,0.203921568627451;0.925490196078431,0.203921568627451,0.203921568627451;0.925490196078431,0.188235294117647,0.188235294117647;0.941176470588235,0.188235294117647,0.188235294117647;0.941176470588235,0.172549019607843,0.172549019607843;0.941176470588235,0.172549019607843,0.172549019607843;0.941176470588235,0.172549019607843,0.172549019607843;0.941176470588235,0.156862745098039,0.156862745098039;0.941176470588235,0.156862745098039,0.156862745098039;0.941176470588235,0.141176470588235,0.141176470588235;0.941176470588235,0.141176470588235,0.141176470588235;0.941176470588235,0.141176470588235,0.141176470588235;0.941176470588235,0.125490196078431,0.125490196078431;0.956862745098039,0.125490196078431,0.125490196078431;0.956862745098039,0.125490196078431,0.125490196078431;0.956862745098039,0.109803921568627,0.109803921568627;0.956862745098039,0.109803921568627,0.109803921568627;0.956862745098039,0.0941176470588235,0.0941176470588235;0.956862745098039,0.0941176470588235,0.0941176470588235;0.956862745098039,0.0941176470588235,0.0941176470588235;0.956862745098039,0.0784313725490196,0.0784313725490196;0.956862745098039,0.0784313725490196,0.0784313725490196;0.956862745098039,0.0627450980392157,0.0627450980392157;0.972549019607843,0.0627450980392157,0.0627450980392157;0.972549019607843,0.0627450980392157,0.0627450980392157;0.972549019607843,0.0470588235294118,0.0470588235294118;0.972549019607843,0.0470588235294118,0.0470588235294118;0.972549019607843,0.0313725490196078,0.0313725490196078;0.972549019607843,0.0313725490196078,0.0313725490196078;0.972549019607843,0.0313725490196078,0.0313725490196078;0.972549019607843,0.0156862745098039,0.0156862745098039;0.972549019607843,0.0156862745098039,0.0156862745098039;0.988235294117647,0,0;0.988235294117647,0,0]];
end