%This program calculates the lagrangian green strain using the NMBS through 
%time for each square elemnt of the NMBS, with some key assumptions. 
%The reference state is set as a perfect undisturbed mesh.
%Calculations start at the bottom left of the mesh. The deformation tensor 
%is calculated using the vectors formed by the bottom left point, the bottom 
%right point, and the top left point. The bottom left point of the deformed 
%mesh is tanslated to its appropriate reference bottom left point. The 
%deformation tensor F is then calculated using the displacement of the ends 
%of the deformed mesh square. F is used %to calculate the right cauchy tenosr 
%C and then the lagrangian green strain E. The eigenvalues of E are the 
%principal strains. 
%
%The connectionsArray is a structure array with the first field being a 
%single trackID representing the bottomLeftDeformed position and the second 
%being an array of trackIDs in the order [bottomRightDeformed, topLeftDeformed,
%topRightDeformed]. 
%
%The locationsArray is a structure array with the first field being the 
%trackID and the second being a matrix with the [x, y] locations of the 
%trackID's corresponding node for different timepoints concatenated vertically. 
function totalStrainCode(connectionsArray, locationsArray)

%The dimensions of the patterned NMBS in microns
squareDim = 100;

%The number of rows in the section being analyzed
gridRows = 5; 

%The number of columns in the section being analyzed
gridCols = 5;

%The number of time points in the image stack being analyzed
numTimePoints = 20; 

%The time point which will be used as the strain state reference
refTimePoint = 1; 

%The output format for the images of the strain state data
plotFileType = 'jpg';

%A flag (1 = true, 0 = false) that will set all of the scales to be equal for each of the output
%images. This can be useful for creating a video of the strain data.
equalColorBars = 1;

%Execute the 2D strain analysis
totalStrainCodeExecutor(connectionsArray, locationsArray, squareDim, gridRows,gridCols,numTimePoints, refTimePoint, plotFileType, equalColorBars)

end

%This script executes the 2D strain analysis
function totalStrainCodeExecutor(connectionsArray, locationsArray, squareDim, gridRows,gridCols,numTimePoints, refTimePoint, plotFileType, equalColorBars)

%Figure width and height for saving
figureWidthHeight = 2000;

%Get the user's desired location for the plots
filePath = uigetdir(matlabroot,'Please Select the Plot Save Location');

%Calculate the strain states throughout time
[ExxMat, EyyMat, ExyMat, PMat, EigVecMat] = timeSeriesStrainData(connectionsArray, locationsArray, squareDim, gridRows,gridCols,numTimePoints, refTimePoint);

%GetPlots for Exx
ExxFigHolder = struct('Fig','Cbar');
maxExxLimits = 0;
parfor i = 1:numTimePoints
    [currentExxFig, currentExxCbar] = plotGrid(connectionsArray,locationsArray, i,ExxMat(i,:),0);
    
    %Set size of figure for saving
    currentExxFig.Position=[0 0 figureWidthHeight figureWidthHeight];
    ExxFigHolder(i).Fig = currentExxFig;
    ExxFigHolder(i).Cbar = currentExxCbar;
    if(i~=1)
        maxExxLimits = max(maxExxLimits,max(currentExxCbar.Limits));
    end
end

%Save plots for Exx
parfor i = 1:numTimePoints
    currentExxFig = ExxFigHolder(i).Fig;

    %Matlab won't update the colorbars if we don't make the figure appear at
    %least once, so if we want to make them all equal for videos then we need
    %this
    if(equalColorBars)
        currentExxFig=figure(currentExxFig);
        currentExxFig.Visible='off';
    end

    currentExxCbar=ExxFigHolder(i).Cbar;

    %If we want equal colorbars across time for a video
    if(equalColorBars)
        currentExxCbar.LimitsMode = 'manual';
        caxis([-maxExxLimits maxExxLimits]);
        currentExxCbar.Limits = [-maxExxLimits, maxExxLimits];
    end

    newFolder = fullfile(filePath, 'Exx');
    [status, msg, msgID]=mkdir(newFolder);
    filePathToSave = fullfile(newFolder, sprintf('Exx%.0f',i));
    saveas(currentExxFig,filePathToSave,plotFileType); 
end

if(equalColorBars)
    close all
end

%Get Plots for Eyy
EyyFigHolder = struct('Fig','Cbar');
maxEyyLimits = 0;
parfor i = 1:numTimePoints
    [currentEyyFig, currentEyyCbar] = plotGrid(connectionsArray,locationsArray, i,EyyMat(i,:),0);
    
    %Set size of figure for saving
    currentEyyFig.Position=[0 0 figureWidthHeight figureWidthHeight];
    EyyFigHolder(i).Fig = currentEyyFig;
    EyyFigHolder(i).Cbar = currentEyyCbar;
    if(i~=1)
        maxEyyLimits = max(maxEyyLimits,max(currentEyyCbar.Limits));
    end
end

%Save plots for Eyy
parfor i = 1:numTimePoints
    currentEyyFig = EyyFigHolder(i).Fig;

    %Matlab won't update the colorbars if we don't make the figure appear at
    %least once, so if we want to make them all equal for videos then we need
    %this
    if(equalColorBars)
        currentEyyFig=figure(currentEyyFig);
        currentEyyFig.Visible='off';
    end

    currentEyyCbar=EyyFigHolder(i).Cbar;
    
    %If we want equal colorbars across time for a video
    if(equalColorBars)
        currentEyyCbar.LimitsMode = 'manual';
        caxis([-maxEyyLimits maxEyyLimits]);
        currentEyyCbar.Limits = [-maxEyyLimits, maxEyyLimits];
    end
    
    newFolder = fullfile(filePath, 'Eyy');
    [status, msg, msgID]=mkdir(newFolder);
    filePathToSave = fullfile(newFolder, sprintf('Eyy%.0f',i));
    saveas(currentEyyFig,filePathToSave,plotFileType); 
end

if(equalColorBars)
    close all
end


%Get Plots for Exy
ExyFigHolder = struct('Fig','Cbar');
maxExyLimits = 0;
parfor i = 1:numTimePoints
    [currentExyFig, currentExyCbar] = plotGrid(connectionsArray,locationsArray, i,ExyMat(i,:),0);
    
    %Set size of figure for saving
    currentExyFig.Position=[0 0 figureWidthHeight figureWidthHeight];
    ExyFigHolder(i).Fig = currentExyFig;
    ExyFigHolder(i).Cbar = currentExyCbar;
    if(i~=1)
        maxExyLimits = max(maxExyLimits,max(currentExyCbar.Limits));
    end
end

%Save plots for Exy
parfor i = 1:numTimePoints
    currentExyFig = ExyFigHolder(i).Fig;
    
    %Matlab won't update the colorbars if we don't make the figure appear at
    %least once, so if we want to make them all equal for videos then we need
    %this
    if(equalColorBars)
        currentExyFig=figure(currentExyFig);
        currentExyFig.Visible='off';
    end
    
    currentExyCbar=ExyFigHolder(i).Cbar;
    
    if(equalColorBars)
        currentExyCbar.LimitsMode = 'manual';
        caxis([-maxExyLimits maxExyLimits]);
        currentExyCbar.Limits = [-maxExyLimits, maxExyLimits];
    end
    newFolder = fullfile(filePath, 'Exy');
    [status, msg, msgID]=mkdir(newFolder);
    filePathToSave = fullfile(newFolder, sprintf('Exy%.0f',i));
    saveas(currentExyFig,filePathToSave,plotFileType); 
end

if(equalColorBars)
    close all
end

%Get Plots for P1
P1FigHolder = struct('Fig','Cbar');
maxP1Limits = 0;
parfor i = 1:numTimePoints
    [currentP1Fig, currentP1Cbar] = plotGrid(connectionsArray,locationsArray, i,PMat(i,:),EigVecMat(2*i-1:2*i,:));
    
    %Set size of figure for saving
    currentP1Fig.Position=[0 0 figureWidthHeight figureWidthHeight];
    P1FigHolder(i).Fig = currentP1Fig;
    P1FigHolder(i).Cbar = currentP1Cbar;
    if(i~=1)
        maxP1Limits = max(maxP1Limits,max(currentP1Cbar.Limits));
    end
end

%Save plots for P1
parfor i = 1:numTimePoints
    currentP1Fig = P1FigHolder(i).Fig;

    %Matlab won't update the colorbars if we don't make the figure appear at
    %least once, so if we want to make them all equal for videos then we need
    %this
    if(equalColorBars)
        currentP1Fig=figure(currentP1Fig);
        currentP1Fig.Visible='off';
    end
    
    currentP1Cbar=P1FigHolder(i).Cbar;
    
    if(equalColorBars)
        currentP1Cbar.LimitsMode = 'manual';
        caxis([-maxP1Limits maxP1Limits]);
        currentP1Cbar.Limits = [-maxP1Limits, maxP1Limits];
    end

    newFolder = fullfile(filePath, 'P1');
    [status, msg, msgID]=mkdir(newFolder);
    filePathToSave = fullfile(newFolder, sprintf('P1%.0f',i));
    saveas(currentP1Fig,filePathToSave,plotFileType); 
end

if(equalColorBars)
    close all
end

%Save Time Series Plots for Exx
[timeSeriesxxFig, cbarxxFig] = timeSeriesPlot(ExxMat, gridRows, gridRows, numTimePoints);
%Set size of figure for saving
timeSeriesxxFig.Position=[0 0 figureWidthHeight figureWidthHeight];
cbarxxFig.Position=[0 0 figureWidthHeight figureWidthHeight];
newFolder = fullfile(filePath, 'TimeSeriesExx');
[status, msg, msgID]=mkdir(newFolder);
filePathToSave = fullfile(newFolder, 'TimeSeriesExx');
saveas(timeSeriesxxFig,filePathToSave,plotFileType);
filePathToSave = fullfile(newFolder, 'TimeSeriesExxColorBar');
saveas(cbarxxFig,filePathToSave,plotFileType);

%Save Time Series Plots for Eyy
[timeSeriesyyFig, cbaryyFig] = timeSeriesPlot(EyyMat, gridRows, gridRows, numTimePoints);
%Set size of figure for saving
timeSeriesyyFig.Position=[0 0 figureWidthHeight figureWidthHeight];
cbaryyFig.Position=[0 0 figureWidthHeight figureWidthHeight];
newFolder = fullfile(filePath, 'TimeSeriesEyy');
[status, msg, msgID]=mkdir(newFolder);
filePathToSave = fullfile(newFolder, 'TimeSeriesEyy');
saveas(timeSeriesyyFig,filePathToSave,plotFileType);
filePathToSave = fullfile(newFolder, 'TimeSeriesEyyColorBar');
saveas(cbaryyFig,filePathToSave,plotFileType);

%Save Time Series Plots for Exy
[timeSeriesxyFig, cbarxyFig] = timeSeriesPlot(ExyMat, gridRows, gridRows, numTimePoints);
%Set size of figure for saving
timeSeriesxyFig.Position=[0 0 figureWidthHeight figureWidthHeight];
cbarxyFig.Position=[0 0 figureWidthHeight figureWidthHeight];
newFolder = fullfile(filePath, 'TimeSeriesExy');
[status, msg, msgID]=mkdir(newFolder);
filePathToSave = fullfile(newFolder, 'TimeSeriesExy');
saveas(timeSeriesxyFig,filePathToSave,plotFileType);
filePathToSave = fullfile(newFolder, 'TimeSeriesExyColorBar');
saveas(cbarxyFig,filePathToSave,plotFileType);

close all
end

%This program calculates the lagrange green strain from the NMBS through 
%time
function [ExxMat, EyyMat, ExyMat, PMat, EigVecMat] = timeSeriesStrainData(connectionsArray, locationsArray, squareDim, gridRows,gridCols,numTimePoints, refTimePoint)

%Preallocate Storage
ExxMat = zeros(numTimePoints,gridRows*gridCols);
EyyMat = zeros(numTimePoints,gridRows*gridCols);
ExyMat = zeros(numTimePoints,gridRows*gridCols);
PMat = zeros(numTimePoints,gridRows*gridCols);
EigVecMat = zeros(2*numTimePoints,gridRows*gridCols);

resultsStorageRef = calcStrainState(connectionsArray, locationsArray, squareDim, refTimePoint,gridRows,gridCols);
index = 1;

%loop through time points
for i = 1:numTimePoints

resultsStorage = calcStrainState(connectionsArray, locationsArray, squareDim, i,gridRows,gridCols);

    for j = 1:length(resultsStorage)

        %Ex Ey Exy
        ExxMat(i,j) = resultsStorage{j}.E(1,1)-resultsStorageRef{j}.E(1,1);
        EyyMat(i,j) = resultsStorage{j}.E(2,2)-resultsStorageRef{j}.E(2,2);
        ExyMat(i,j) = resultsStorage{j}.E(1,2)-resultsStorageRef{j}.E(1,2);
        currentE = resultsStorage{j}.E-resultsStorageRef{j}.E;

        %Max Principal Strain and Vector
        [V,D] = eig(currentE);
        [maxP, ind] = max(sort(diag(D)));
        PMat(i,j) = maxP;
        EigVecMat(index:index+1,j)=V(ind,:);
    end
    
index = index+2;
end
end

%This subscript calculates the strain state for a single timepoint
function resultsStorage = calcStrainState(connectionsArray, locationsArray, squareDim, timePoint,gridRows,gridCols)

[r, c] = size(connectionsArray);
numConnections = max([r,c]);

resultsStorage = cell(numConnections,1);

[r2, c2] = size(locationsArray);
numLocations = max([r2,c2]);

%pull out the location trackID array
locationInOrder = zeros(numLocations,1);
for i = 1:numLocations
   locationInOrder(i) = locationsArray(i).number;
end

%Generate and store the reference locations
refLocsCell = cell(25,1);
index=1;
for i = 1:gridRows
    for j =1:gridCols
        currentBottomLeft = [(j-1)*squareDim, (i-1)*squareDim];
        currentBottomRight = [(j)*squareDim, (i-1)*squareDim];
        currentTopLeft = [(j-1)*squareDim, (i)*squareDim];
        refLocsCell{index} = [currentBottomLeft;currentBottomRight;currentTopLeft];
        index = index+1;
    end
end

%Calculate the deformation, right cauchy, and lagrange green strain tensors
for i = 1:numConnections
    
    %Get the trackID for the relevant locations
    bottomLeft = connectionsArray(i).Num;
    bottomRight = connectionsArray(i).connections(1);
    topLeft = connectionsArray(i).connections(2);
    
    %Get the deformed locations
    bottomLeftLocs = locationsArray(find(locationInOrder == bottomLeft)).locations(timePoint,:);
    bottomRightLocs = locationsArray(find(locationInOrder == bottomRight)).locations(timePoint,:);
    topLeftLocs = locationsArray(find(locationInOrder == topLeft)).locations(timePoint,:);
   
    %Prepare Inputs
    defLocs = [bottomLeftLocs;bottomRightLocs;topLeftLocs];
    refLocs = refLocsCell{i};
    
    %Calculate the tensors
    [F, C, E] = calcTensors(refLocs, defLocs, squareDim);
    
    %store the tensors
    currentResults.E = E;
    currentResults.C = C;
    currentResults.F = F;
    resultsStorage{i} = currentResults;
end



end

%Calculate the deformation, Right cauchy, and lagrange green strain
%tensors for a deformed section of an NMBS
function [F, C, E] = calcTensors(refLocs, defLocs, squareDim)

%Pull out the reference locations
refBotLeft = refLocs(1,:);
refBotRight = refLocs(2,:);
refTopLeft = refLocs(3,:);

%Pull out the deformed locations
defBotLeft = defLocs(1,:);
defBotRight = defLocs(2,:);
defTopLeft = defLocs(3,:);

%Determine the transformation between the reference bottom left and the
%deformed bottom left
translationDistanceX = defBotLeft(1)-refBotLeft(1);
translationDistanceY = defBotLeft(2)-refBotLeft(2);

%Translate the deformed locations
defBotLeftTrans = defBotLeft-[translationDistanceX,translationDistanceY];
defBotRightTrans = defBotRight-[translationDistanceX,translationDistanceY];
defTopLeftTrans = defTopLeft-[translationDistanceX,translationDistanceY];

%Calculate the deformations
botRightXDeformation = (defBotRightTrans(1)-defBotLeftTrans(1))/squareDim;
botRightYDeformation = (defBotRightTrans(2)-defBotLeftTrans(2))/squareDim;
topLeftXDeformation = (defTopLeftTrans(1)-defBotLeftTrans(1))/squareDim;
topLeftYDeformation = (defTopLeftTrans(2)-defBotLeftTrans(2))/squareDim;

%Calculate the deformation, Right cauchy, and lagrange green strain tensors
F = [botRightXDeformation topLeftXDeformation; botRightYDeformation topLeftYDeformation];
C = F'*F;
E = .5.*(C-eye(2));
end

%This function plots the deformed NMBS at a certain timepoint with the
%square elements filled with the appropriate supplied strain data (e.g. Ex)
function [gridFig, cbar] = plotGrid(connectionsArray,locationsArray, timePoint, strainInput, eigenvecsIn)

%Generate the figure    
gridFig = figure('visible','off');
gridFigAxes = gca;
gridFigAxes.LineWidth = 4;
gridFigAxes.FontSize = 32;
hold on
[r2, c2] = size(locationsArray);
numLocations = max([r2,c2]);

%pull out the location trackID array
locationInOrder = zeros(numLocations,1);
for i = 1:numLocations
   locationInOrder(i) = locationsArray(i).number;
end

%Move through the connectionsArray to generate a patch for the plot
for i = 1:length(connectionsArray)
    
    %Get the trackID for the relevant locations
    bottomLeft = connectionsArray(i).Num;
    bottomRight = connectionsArray(i).connections(1);
    topLeft = connectionsArray(i).connections(2);
    topRight = connectionsArray(i).connections(3);
    
    %Get the deformed locations
    bottomLeftLocs = locationsArray(find(locationInOrder == bottomLeft)).locations(timePoint,:);
    bottomRightLocs = locationsArray(find(locationInOrder == bottomRight)).locations(timePoint,:);
    topLeftLocs = locationsArray(find(locationInOrder == topLeft)).locations(timePoint,:);
    topRightLocs = locationsArray(find(locationInOrder == topRight)).locations(timePoint,:);
    
    %Define patch
    x = [bottomRightLocs(1), bottomLeftLocs(1), topLeftLocs(1), topRightLocs(1), bottomRightLocs(1)];
    y = [bottomRightLocs(2), bottomLeftLocs(2), topLeftLocs(2), topRightLocs(2), bottomRightLocs(2)];
    p=patch(x,y,strainInput(i));
    p.LineWidth = 3;
    
    %If eigenvectors are supplied quiver them
    if(numel(eigenvecsIn)>1)
        q = quiver(mean(x(1:4)), mean(y(1:4)), eigenvecsIn(1,i), eigenvecsIn(2,i));
        q.Color = 'white';
        q.LineWidth = 3;
        q.AutoScaleFactor = 30;
        q.MaxHeadSize = 20;
    end
end

% Phase colormap
colormap(getPhase);
cbar = colorbar;

%Creating the colorbar  based upon calculated range
cbar.LimitsMode = 'auto';
Limits = cbar.Limits;
LowLim = -max(abs(Limits));
HighLim = max(abs(Limits));
caxis([LowLim HighLim]);
cbar.Limits = [LowLim HighLim];
end

%This function plots the time series data for a specific strain for each of
%the square elements of the NMBS in subplots starting at the top left
function [timeSeriesFig,cbarFig] = timeSeriesPlot(strainData, numRows, numCols, numTimePoints)

%Generate the figure and pull in the colormap
timeSeriesFig = figure;
colormap(getPhase);
axisScale = 0;

%Iterate through the NMBS square elements
for i = 1:numRows*numCols
    
    %Pull out the strain data for this NMBS square element
    currentData = strainData(:,i);
    currentAxisScale = max(abs(currentData));
    if(currentAxisScale>axisScale)
        axisScale = currentAxisScale;
    end
    
    %Generate a subplot in the correct location
    subplot(numRows,numCols,i);
    
    %Plot the strain data with a zero line, markers, and a connecting line
    plot(linspace(0,numTimePoints,numTimePoints),currentData,'LineWidth',1,'Color','k')
    hold on
    scatter(linspace(0,numTimePoints,numTimePoints),currentData,600,currentData,'filled')
    plot(linspace(0,numTimePoints,numTimePoints),zeros(1,numTimePoints),'LineStyle','--','Color','k','LineWidth',2)
    
    %Format the plot 
    box off
    ax = gca;
    ax.LineWidth = 4;
    
    %Option to not have x and y labels
    %ax.YTick=[];
    %ax.XTick=[];
end

%Scale all the axes and colorbars the same
for i = 1:numRows*numCols
    subplot(numRows,numCols,i);
    axis([0 numTimePoints -1.1*axisScale 1.1*axisScale]);
    cbar = colorbar;
    cbar.Limits = [-1.1*axisScale 1.1*axisScale];
    caxis([-1.1*axisScale 1.1*axisScale]);
    colorbar('off');
end

%Make a colorbar figure
cbarFig = figure;
colormap(getPhase);
cbar = colorbar;
caxis([-1.1*axisScale 1.1*axisScale]);
cbar.Limits = [-1.1*axisScale 1.1*axisScale];

%Hide the two figures
timeSeriesFig.Visible = 'off';
cbarFig.Visible = 'off';

end

%Custom phase colormap
function custcm = getPhase
custcm = [[0,0,0.988235294117647;0,0,0.988235294117647;0,0,0.988235294117647;0.0156862745098039,0.0156862745098039,0.988235294117647;0.0156862745098039,0.0156862745098039,0.988235294117647;0.0156862745098039,0.0156862745098039,0.988235294117647;0.0313725490196078,0.0313725490196078,0.988235294117647;0.0313725490196078,0.0313725490196078,0.988235294117647;0.0470588235294118,0.0470588235294118,0.988235294117647;0.0470588235294118,0.0470588235294118,0.988235294117647;0.0470588235294118,0.0470588235294118,0.972549019607843;0.0627450980392157,0.0627450980392157,0.972549019607843;0.0627450980392157,0.0627450980392157,0.972549019607843;0.0784313725490196,0.0784313725490196,0.972549019607843;0.0784313725490196,0.0784313725490196,0.972549019607843;0.0784313725490196,0.0784313725490196,0.972549019607843;0.0941176470588235,0.0941176470588235,0.972549019607843;0.0941176470588235,0.0941176470588235,0.972549019607843;0.109803921568627,0.109803921568627,0.972549019607843;0.109803921568627,0.109803921568627,0.972549019607843;0.109803921568627,0.109803921568627,0.956862745098039;0.125490196078431,0.125490196078431,0.956862745098039;0.125490196078431,0.125490196078431,0.956862745098039;0.141176470588235,0.141176470588235,0.956862745098039;0.141176470588235,0.141176470588235,0.956862745098039;0.141176470588235,0.141176470588235,0.956862745098039;0.156862745098039,0.156862745098039,0.956862745098039;0.156862745098039,0.156862745098039,0.956862745098039;0.172549019607843,0.172549019607843,0.956862745098039;0.172549019607843,0.172549019607843,0.956862745098039;0.172549019607843,0.172549019607843,0.941176470588235;0.188235294117647,0.188235294117647,0.941176470588235;0.188235294117647,0.188235294117647,0.941176470588235;0.203921568627451,0.203921568627451,0.941176470588235;0.203921568627451,0.203921568627451,0.941176470588235;0.203921568627451,0.203921568627451,0.941176470588235;0.219607843137255,0.219607843137255,0.941176470588235;0.219607843137255,0.219607843137255,0.941176470588235;0.235294117647059,0.235294117647059,0.941176470588235;0.235294117647059,0.235294117647059,0.925490196078431;0.235294117647059,0.235294117647059,0.925490196078431;0.250980392156863,0.250980392156863,0.925490196078431;0.250980392156863,0.250980392156863,0.925490196078431;0.266666666666667,0.266666666666667,0.925490196078431;0.266666666666667,0.266666666666667,0.925490196078431;0.266666666666667,0.266666666666667,0.925490196078431;0.282352941176471,0.282352941176471,0.925490196078431;0.282352941176471,0.282352941176471,0.925490196078431;0.298039215686275,0.298039215686275,0.925490196078431;0.298039215686275,0.298039215686275,0.909803921568627;0.298039215686275,0.298039215686275,0.909803921568627;0.313725490196078,0.313725490196078,0.909803921568627;0.313725490196078,0.313725490196078,0.909803921568627;0.329411764705882,0.329411764705882,0.909803921568627;0.329411764705882,0.329411764705882,0.909803921568627;0.329411764705882,0.329411764705882,0.909803921568627;0.345098039215686,0.345098039215686,0.909803921568627;0.345098039215686,0.345098039215686,0.909803921568627;0.360784313725490,0.360784313725490,0.909803921568627;0.360784313725490,0.360784313725490,0.894117647058824;0.360784313725490,0.360784313725490,0.894117647058824;0.376470588235294,0.376470588235294,0.894117647058824;0.376470588235294,0.376470588235294,0.894117647058824;0.376470588235294,0.376470588235294,0.894117647058824;0.392156862745098,0.392156862745098,0.894117647058824;0.392156862745098,0.392156862745098,0.894117647058824;0.407843137254902,0.407843137254902,0.894117647058824;0.407843137254902,0.407843137254902,0.894117647058824;0.407843137254902,0.407843137254902,0.878431372549020;0.423529411764706,0.423529411764706,0.878431372549020;0.423529411764706,0.423529411764706,0.878431372549020;0.439215686274510,0.439215686274510,0.878431372549020;0.439215686274510,0.439215686274510,0.878431372549020;0.439215686274510,0.439215686274510,0.878431372549020;0.454901960784314,0.454901960784314,0.878431372549020;0.454901960784314,0.454901960784314,0.878431372549020;0.470588235294118,0.470588235294118,0.878431372549020;0.470588235294118,0.470588235294118,0.878431372549020;0.470588235294118,0.470588235294118,0.862745098039216;0.486274509803922,0.486274509803922,0.862745098039216;0.486274509803922,0.486274509803922,0.862745098039216;0.501960784313726,0.501960784313726,0.862745098039216;0.501960784313726,0.501960784313726,0.862745098039216;0.501960784313726,0.501960784313726,0.862745098039216;0.517647058823530,0.517647058823530,0.862745098039216;0.517647058823530,0.517647058823530,0.862745098039216;0.533333333333333,0.533333333333333,0.862745098039216;0.533333333333333,0.533333333333333,0.862745098039216;0.533333333333333,0.533333333333333,0.847058823529412;0.549019607843137,0.549019607843137,0.847058823529412;0.549019607843137,0.549019607843137,0.847058823529412;0.564705882352941,0.564705882352941,0.847058823529412;0.564705882352941,0.564705882352941,0.847058823529412;0.564705882352941,0.564705882352941,0.847058823529412;0.580392156862745,0.580392156862745,0.847058823529412;0.580392156862745,0.580392156862745,0.847058823529412;0.596078431372549,0.596078431372549,0.847058823529412;0.596078431372549,0.596078431372549,0.831372549019608;0.596078431372549,0.596078431372549,0.831372549019608;0.611764705882353,0.611764705882353,0.831372549019608;0.611764705882353,0.611764705882353,0.831372549019608;0.627450980392157,0.627450980392157,0.831372549019608;0.627450980392157,0.627450980392157,0.831372549019608;0.627450980392157,0.627450980392157,0.831372549019608;0.643137254901961,0.643137254901961,0.831372549019608;0.643137254901961,0.643137254901961,0.831372549019608;0.658823529411765,0.658823529411765,0.831372549019608;0.658823529411765,0.658823529411765,0.815686274509804;0.658823529411765,0.658823529411765,0.815686274509804;0.674509803921569,0.674509803921569,0.815686274509804;0.674509803921569,0.674509803921569,0.815686274509804;0.690196078431373,0.690196078431373,0.815686274509804;0.690196078431373,0.690196078431373,0.815686274509804;0.690196078431373,0.690196078431373,0.815686274509804;0.705882352941177,0.705882352941177,0.815686274509804;0.705882352941177,0.705882352941177,0.815686274509804;0.721568627450980,0.721568627450980,0.815686274509804;0.721568627450980,0.721568627450980,0.800000000000000;0.721568627450980,0.721568627450980,0.800000000000000;0.737254901960784,0.737254901960784,0.800000000000000;0.737254901960784,0.737254901960784,0.800000000000000;0.752941176470588,0.752941176470588,0.800000000000000;0.752941176470588,0.752941176470588,0.800000000000000;0.752941176470588,0.752941176470588,0.800000000000000;0.768627450980392,0.768627450980392,0.800000000000000;0.768627450980392,0.768627450980392,0.800000000000000;0.784313725490196,0.784313725490196,0.784313725490196;0.784313725490196,0.784313725490196,0.784313725490196;0.784313725490196,0.784313725490196,0.784313725490196;0.784313725490196,0.768627450980392,0.768627450980392;0.784313725490196,0.768627450980392,0.768627450980392;0.784313725490196,0.768627450980392,0.768627450980392;0.784313725490196,0.752941176470588,0.752941176470588;0.784313725490196,0.752941176470588,0.752941176470588;0.784313725490196,0.737254901960784,0.737254901960784;0.784313725490196,0.737254901960784,0.737254901960784;0.800000000000000,0.737254901960784,0.737254901960784;0.800000000000000,0.721568627450980,0.721568627450980;0.800000000000000,0.721568627450980,0.721568627450980;0.800000000000000,0.705882352941177,0.705882352941177;0.800000000000000,0.705882352941177,0.705882352941177;0.800000000000000,0.705882352941177,0.705882352941177;0.800000000000000,0.690196078431373,0.690196078431373;0.800000000000000,0.690196078431373,0.690196078431373;0.800000000000000,0.674509803921569,0.674509803921569;0.800000000000000,0.674509803921569,0.674509803921569;0.815686274509804,0.674509803921569,0.674509803921569;0.815686274509804,0.658823529411765,0.658823529411765;0.815686274509804,0.658823529411765,0.658823529411765;0.815686274509804,0.658823529411765,0.658823529411765;0.815686274509804,0.643137254901961,0.643137254901961;0.815686274509804,0.643137254901961,0.643137254901961;0.815686274509804,0.627450980392157,0.627450980392157;0.815686274509804,0.627450980392157,0.627450980392157;0.815686274509804,0.627450980392157,0.627450980392157;0.815686274509804,0.611764705882353,0.611764705882353;0.831372549019608,0.611764705882353,0.611764705882353;0.831372549019608,0.596078431372549,0.596078431372549;0.831372549019608,0.596078431372549,0.596078431372549;0.831372549019608,0.596078431372549,0.596078431372549;0.831372549019608,0.580392156862745,0.580392156862745;0.831372549019608,0.580392156862745,0.580392156862745;0.831372549019608,0.564705882352941,0.564705882352941;0.831372549019608,0.564705882352941,0.564705882352941;0.831372549019608,0.564705882352941,0.564705882352941;0.831372549019608,0.549019607843137,0.549019607843137;0.847058823529412,0.549019607843137,0.549019607843137;0.847058823529412,0.533333333333333,0.533333333333333;0.847058823529412,0.533333333333333,0.533333333333333;0.847058823529412,0.533333333333333,0.533333333333333;0.847058823529412,0.517647058823530,0.517647058823530;0.847058823529412,0.517647058823530,0.517647058823530;0.847058823529412,0.517647058823530,0.517647058823530;0.847058823529412,0.501960784313726,0.501960784313726;0.847058823529412,0.501960784313726,0.501960784313726;0.847058823529412,0.486274509803922,0.486274509803922;0.862745098039216,0.486274509803922,0.486274509803922;0.862745098039216,0.486274509803922,0.486274509803922;0.862745098039216,0.470588235294118,0.470588235294118;0.862745098039216,0.470588235294118,0.470588235294118;0.862745098039216,0.454901960784314,0.454901960784314;0.862745098039216,0.454901960784314,0.454901960784314;0.862745098039216,0.454901960784314,0.454901960784314;0.862745098039216,0.439215686274510,0.439215686274510;0.862745098039216,0.439215686274510,0.439215686274510;0.862745098039216,0.423529411764706,0.423529411764706;0.878431372549020,0.423529411764706,0.423529411764706;0.878431372549020,0.423529411764706,0.423529411764706;0.878431372549020,0.407843137254902,0.407843137254902;0.878431372549020,0.407843137254902,0.407843137254902;0.878431372549020,0.392156862745098,0.392156862745098;0.878431372549020,0.392156862745098,0.392156862745098;0.878431372549020,0.392156862745098,0.392156862745098;0.878431372549020,0.376470588235294,0.376470588235294;0.878431372549020,0.376470588235294,0.376470588235294;0.894117647058824,0.376470588235294,0.376470588235294;0.894117647058824,0.360784313725490,0.360784313725490;0.894117647058824,0.360784313725490,0.360784313725490;0.894117647058824,0.345098039215686,0.345098039215686;0.894117647058824,0.345098039215686,0.345098039215686;0.894117647058824,0.345098039215686,0.345098039215686;0.894117647058824,0.329411764705882,0.329411764705882;0.894117647058824,0.329411764705882,0.329411764705882;0.894117647058824,0.313725490196078,0.313725490196078;0.894117647058824,0.313725490196078,0.313725490196078;0.909803921568627,0.313725490196078,0.313725490196078;0.909803921568627,0.298039215686275,0.298039215686275;0.909803921568627,0.298039215686275,0.298039215686275;0.909803921568627,0.282352941176471,0.282352941176471;0.909803921568627,0.282352941176471,0.282352941176471;0.909803921568627,0.282352941176471,0.282352941176471;0.909803921568627,0.266666666666667,0.266666666666667;0.909803921568627,0.266666666666667,0.266666666666667;0.909803921568627,0.266666666666667,0.266666666666667;0.909803921568627,0.250980392156863,0.250980392156863;0.925490196078431,0.250980392156863,0.250980392156863;0.925490196078431,0.235294117647059,0.235294117647059;0.925490196078431,0.235294117647059,0.235294117647059;0.925490196078431,0.235294117647059,0.235294117647059;0.925490196078431,0.219607843137255,0.219607843137255;0.925490196078431,0.219607843137255,0.219607843137255;0.925490196078431,0.203921568627451,0.203921568627451;0.925490196078431,0.203921568627451,0.203921568627451;0.925490196078431,0.203921568627451,0.203921568627451;0.925490196078431,0.188235294117647,0.188235294117647;0.941176470588235,0.188235294117647,0.188235294117647;0.941176470588235,0.172549019607843,0.172549019607843;0.941176470588235,0.172549019607843,0.172549019607843;0.941176470588235,0.172549019607843,0.172549019607843;0.941176470588235,0.156862745098039,0.156862745098039;0.941176470588235,0.156862745098039,0.156862745098039;0.941176470588235,0.141176470588235,0.141176470588235;0.941176470588235,0.141176470588235,0.141176470588235;0.941176470588235,0.141176470588235,0.141176470588235;0.941176470588235,0.125490196078431,0.125490196078431;0.956862745098039,0.125490196078431,0.125490196078431;0.956862745098039,0.125490196078431,0.125490196078431;0.956862745098039,0.109803921568627,0.109803921568627;0.956862745098039,0.109803921568627,0.109803921568627;0.956862745098039,0.0941176470588235,0.0941176470588235;0.956862745098039,0.0941176470588235,0.0941176470588235;0.956862745098039,0.0941176470588235,0.0941176470588235;0.956862745098039,0.0784313725490196,0.0784313725490196;0.956862745098039,0.0784313725490196,0.0784313725490196;0.956862745098039,0.0627450980392157,0.0627450980392157;0.972549019607843,0.0627450980392157,0.0627450980392157;0.972549019607843,0.0627450980392157,0.0627450980392157;0.972549019607843,0.0470588235294118,0.0470588235294118;0.972549019607843,0.0470588235294118,0.0470588235294118;0.972549019607843,0.0313725490196078,0.0313725490196078;0.972549019607843,0.0313725490196078,0.0313725490196078;0.972549019607843,0.0313725490196078,0.0313725490196078;0.972549019607843,0.0156862745098039,0.0156862745098039;0.972549019607843,0.0156862745098039,0.0156862745098039;0.988235294117647,0,0;0.988235294117647,0,0]];
end